-- Covert Operations Mission_Menu_Popup (Main Covert Ops module - parent for most of the "working" files)
-- Author: sman	
-- DateCreated: 12/21/17 9:07:48 AM
--------------------------------------------------------------

-- ===============================================================================================
--		Globals

g_CurrentPlayer = 0																			-- This variable is a global value that always represents the current player - more accurate/reliable than GetActivePlayer

function OnPlayerDoTurn1( iPlayer )															-- Update the g_CurrentPlayer value every time the player turn changes
  g_CurrentPlayer = iPlayer																	-- The global g_CurrentPlayer should ALWAYS have the actual active player;							
end																							-- And be visible to lower-level/lower-numbered functions/files
GameEvents.PlayerDoTurn.Add( OnPlayerDoTurn1 )



-- ===============================================================================================
--		includes

include("IconSupport")
include("InstanceManager")
include("InfoTooltipInclude")

include("00_Globals.lua")
include("60_Mission_Support_Functions.lua")													-- Various support functions used by all of the following include files/this file
include("20_Process_Mission_Results.lua")
include("30_Process_International_Responses.lua")											-- Determines international community responses for all covert operations
include("40_Process_Operator_Destiny.lua")													-- Determines operator fate for the human player only; AI doesn't use this
include("50_Process_Recon_Results.lua")														-- Builds the Recon Report popup/notification for the human player only; AI doesn't use Recon mission
include("25_Process_AI_Mission_Results.lua")												-- Performs AI missions, in a similar way that the human player does, without UI features
																							-- Not sure if I should move the "25 process ai" include statement to the bottom of this file?

local g_ItemManager = InstanceManager:new( "ItemInstance", "Button", Controls.ItemStack )
local g_SelectedMission = ABORT_MISSION

-- ===============================================================================================
--		Support Functions

function DisplayPopup()																		-- Function that builds the popup and displays it
	local pPlayer = Players[Game.GetActivePlayer()]											-- Get current player; ok to use GetActivePlayer as only the human player can use this popup
	local pUnit = UI.GetHeadSelectedUnit()													-- A unit must be selected for the Launch Button to active-which must be pressed for this popup to launch
	local pPlot = pUnit:GetPlot()															-- Get location of selected unit
	
	BuildValidMissionTable(pPlayer, pUnit, pPlot)											-- Function builds a table of the missions this unit is allowed to launch, based on its location, etc.
	g_ItemManager:ResetInstances()															-- Clear instance, start with fresh menu for this one

	for iNum = 1, g_NumMissions do															-- Cycle through all possible missions
		
		if g_ValidMissions[iNum][1] == YES then												-- Only load "valid" missions into the menu
		
			local controlTable = g_ItemManager:GetInstance()			
						
			IconHookup( iNum + 15, 64, "SMAN_COVERT_OPS_ACTION_ATLAS", controlTable.Icon64 )	-- Silver Icon at top of popup
			--IconHookup( iNum - 1, 64, "SMAN_COVERT_OPS_ACTION_ATLAS", controlTable.Icon64 )	-- Gold Icon at top of popup

			local keyMissionTitleString = "TXT_KEY_COVERT_OPS_MISSION_TITLE_" .. iNum		-- Get the correct text strings for this mission type
			local keyMissionBodyString = "TXT_KEY_COVERT_OPS_MISSION_BODY_" .. iNum
		
			controlTable.Name:LocalizeAndSetText(keyMissionTitleString)
			controlTable.Button:SetToolTipString(Locale.ConvertTextKey(keyMissionBodyString, g_MissionList[iNum][2], g_MissionList[iNum][3], g_MissionList[iNum][4]))
			controlTable.Button:SetDisabled(false)				
		
			local selectionAnim = controlTable.SelectionAnim
			controlTable.Button:SetVoid1(iNum)
			controlTable.Button:RegisterCallback(Mouse.eLClick, function()  g_SelectedMission = iNum	DisplayPopup() end)		
		
			if g_SelectedMission then
				if g_SelectedMission == iNum then
					controlTable.SelectionAnim:SetHide(false)
				else
					controlTable.SelectionAnim:SetHide(true)
				end
			else
				controlTable.SelectionAnim:SetHide(true)
			end
		end 
	end

	Controls.ItemStack:CalculateSize()
	Controls.ItemStack:ReprocessAnchoring()
	Controls.ItemScrollPanel:CalculateInternalSize()
	Controls.ItemScrollPanel:SetScrollValue(0)
	Controls.ConfirmButton:SetDisabled(true)

	ContextPtr:SetHide(false)
	if g_SelectedMission then
		Controls.ConfirmButton:SetDisabled(false)
	end	
end


function OnConfirm()																		-- Process actions once the player has selected a mission
    ContextPtr:SetHide(true)																-- Hide the Menu Popup window
	--print("========> Selected mission: " .. g_SelectedMission .. "   for mission: " .. g_MissionList[g_SelectedMission][1])
	ProcessMission (g_SelectedMission)														-- Process the selected mission
	g_SelectedMission = ABORT_MISSION														-- Sets the default menu position to "Abort Mission" for the next time the popup unhides
end
Controls.ConfirmButton:RegisterCallback( Mouse.eLClick, OnConfirm )


function ShowCovertMissionPopup()															-- LUA Event that calls the function that will unhide the popup
	DisplayPopup()
end 
LuaEvents.CovertMissionPopup.Add( ShowCovertMissionPopup )


function OnActivePlayerChanged( iActivePlayer, iPrevActivePlayer )							-- Ensure popup is hidden when active player changes
	if (not ContextPtr:IsHidden()) then														-- If the popup is not hidden, then
		ContextPtr:SetHide(true)															-- Hide the popup
	end
end
Events.GameplaySetActivePlayer.Add(OnActivePlayerChanged)

--UIManager:QueuePopup(ContextPtr, PopupPriority.SocialPolicy)
UIManager:DequeuePopup(ContextPtr)