-- Covert Operartions 20_Process_Mission_Outcomes.lua     MAIN/TOP-LEVEL FUNCTION
-- Author: sman
-- DateCreated: 01/08/18 9:18:23 AM
--------------------------------------------------------------

-- ===============================================================================================
--		Globals


-- ===============================================================================================
--		includes


-- ===============================================================================================
--		Support Functions

function GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)				-- Determines if the covert mission was successful - returns YES/NO
	local iMissionResult = NO																-- Default result is mission failed
	local iBuildingInfluence = GetBuildingInfluence(pTargetCity)							-- Get how many defensive buildings are in the target city

	--  Adjust results based on level (competence) of Operator	
	local iAdjustmentByOperatorLevel = math.floor((iAttackingOperatorLevel - 1) * 5)		-- Level 1 gets no bonus, Level 4 gets 20% better chance of positive result
	
	-- Get mission success result
	local iAdjustmentLimit = math.floor( (g_MissionList[missionNumber][2] + iAdjustmentByOperatorLevel) * ((100 - iBuildingInfluence) / 100) )	-- Building influence if a num from 0..68, representing what % the chance of success is lowered by certain buildings in the city
	if iAdjustmentLimit < g_iMinimumMissionAdjustment then iAdjustmentLimit = g_iMinimumMissionAdjustment end	-- ensure there is at least some chance of success
	--print("Mission adjustments. Normal success percent: " .. g_MissionList[missionNumber][2] .. "   Operator Adjustment: " .. iAdjustmentByOperatorLevel .. "   Building Adjustment: " .. iBuildingInfluence .. "    Final Adjustment Limit: " .. iAdjustmentLimit)
	if math.random(1, 100) < iAdjustmentLimit then											-- Was Covert Mission successful?  Based on adjusted mission probability for the basic mission (g_MissionList[missionNumber][2])
		iMissionResult = YES
	end			
	return iMissionResult
end


function GetMissionResult(tMissionResultData, missionNumber, pPlayer, pUnit, currentEra, pTargetPlayer, pTargetCity)	-- Function determines mission results
	--print("==> Inside GetMissionResult.  Mission: " .. g_MissionList[missionNumber][1] .. "    Mission Number: " .. missionNumber)

	local iAttackingOperatorLevel = 1														-- Initial value for the level of the operator launching an attack: should be changed for either AI or human player with other values
	local pPlot = nil																		-- Will be a plot where the covert operation is set to occur: either a city or an adjacent unit
	local NotificationType = 1																-- Set up defaults for the AddNotification function
	local NotificationUnit = nil															-- Most notification will display the Operative unit, but not always
	local NotificationStringFading = ""														-- Clear notification strings for later appending
	local NotificationStringLong = ""
	local NotificationX = -1																-- The Notification will center on these points
	local NotificationY = -1
	local MissionResult = NO																-- Ensure initial/default results are clear/negative
	
	-- ======================= Adjustments for AI											-- Alter certain default values when attacking unit is AI (and therefore not an actual unit)	
	if not pPlayer:IsHuman() then 	
		--NotificationUnit = ???															-- This needs some thought - what icon to use on notifications when there is no unit?		
		iAttackingOperatorLevel = g_MissionList[missionNumber][6]							-- Sets the value to the Level value of the mission itself
		NotificationX = pTargetCity:GetX()													-- Notification camera position will default to target city, not the attacking unit
		NotificationY = pTargetCity:GetY()
		pPlot = Map.GetPlot(NotificationX, NotificationY)									-- pPlot is the target city plot
	else
		iAttackingOperatorLevel = GameInfo.Units[pUnit:GetUnitType()].IsCovertOperatorUnit or 1	-- Attacking unit CO level - should default to level 1 if pUnit is nil (for AI attacks)
		pPlot = pUnit:GetPlot()																-- Get that operator's plot	
		NotificationUnit = pUnit															-- Usually, notification will display the Operative unit
		NotificationX = pUnit:GetX() or -1													-- Get unit's "original" X/Y - so notification points there even if unit is killed
		NotificationY = pUnit:GetY() or -1
	end
		
	-- ==================================================================================	Individual Mission Effects		
	-- ======================		Mission: Conduct Recon		=========================
	if missionNumber == CONDUCT_RECON then
		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results; probably human only - AI shouldn't launch this mission
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY = pTargetCity:GetY()
					
		if MissionResult == YES then														-- Conduct Recon mission was successful			
			local reconHeader, reconReport, reconSummary = ProcessReconMissionResults(pUnit, pTargetCity)	-- Get the results of the recon mission - even if it's a failure			
			--NotificationType = 1															-- Build the notification strings
			NotificationStringFading = "Successful reconnaisance near " .. pTargetCity:GetName() .. "!"  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " conducted a successful reconnaissance mission near " .. pTargetCity:GetName() .. "![NEWLINE][NEWLINE]Report Summary: " .. reconSummary .. "[NEWLINE][NEWLINE]"						
			LuaEvents.ProcessReconReport(reconHeader, reconReport)							-- Display Recon Report popup			
		else																				-- Poison mission failed; only notification actions are taken
			--NotificationType = 1
			NotificationStringFading = "Attempted reconnaissance near " .. pTargetCity:GetName() .. " failed."  
			NotificationStringLong = "An attempt to conduct reconnaissance near " .. pTargetCity:GetName() .. " failed."
		end

	-- ======================		Mission: Poison Water		=========================
	elseif missionNumber == POISON_WATER then																
		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves Notification focus to attacked city
		NotificationY = pTargetCity:GetY()
		
		if MissionResult == YES then														-- Poison mission was successful
			local popLoss = math.random(1, currentEra + 1)									-- Get population to be lost, influenced by Attacker civ's Era (higher losses in later Eras)
			local targetCityPop = math.floor(pTargetCity:GetPopulation())					-- Get current target city pop			
			if popLoss >= targetCityPop then popLoss = targetCityPop - 1 end				-- Prevent pop from falling below 1			
			local newPop = targetCityPop - popLoss											-- Determine new poplation for target city			
			pTargetCity:SetPopulation(newPop, true)											-- Set target city's pop to the new (lower) number
						
			if (not pPlayer:IsHuman()) and (pTargetPlayer:IsHuman()) then 					-- Set the notification for a successful AI attack; change Notification type from human attacks
				NotificationType = 5
			end
			NotificationStringFading = "The wells near " .. pTargetCity:GetName() .. " were poisoned!"  
			NotificationStringLong = "The wells near " .. pTargetCity:GetName() .. " were poisoned!  The city lost " .. popLoss .. " population."

		else																				-- Poison mission failed; only notification actions are taken

			if (not pPlayer:IsHuman()) and (pTargetPlayer:IsHuman()) then 					-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationType = 4
			end
			NotificationStringFading = "An attempt to poison the wells near " .. pTargetCity:GetName() .. " failed."  
			NotificationStringLong = "An attempt to poison the wells near " .. pTargetCity:GetName() .. " failed. "
		end

	-- ======================		Mission: Spread Rumors and Propaganda		===========================
	elseif (missionNumber == SPREAD_RUMORS) or (missionNumber == SPREAD_PROPAGANDA) then															

		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY = pTargetCity:GetY()
						
		if MissionResult == YES then														-- Spread Rumors mission was successful
			local missionType = "Rumors"
			--local targetPlayer = Players[pTargetCity:GetOwner()]							-- Returns player object for owner of target city FIXX
			local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or "Enemy"		-- Target city owner civ adjective
			local lowerRange = 10															-- Range for Rumors is 10..25
			local higherRange = 25
		
			if missionNumber == SPREAD_PROPAGANDA then										-- Range for Propaganda is 5..66
				missionType = "Propaganda"
				lowerRange = 5
				higherRange = 66
			end

			local adjustmentCoefficient = math.random(lowerRange, higherRange)				-- Determine where in the range we set the random adjustment
			local oldCulture = pTargetPlayer:GetJONSCulture()								-- Get the existing Culture amount
			local newCulture = math.ceil( oldCulture * (adjustmentCoefficient/100) )		-- Determine new Culture amounts as the old amount * (adjustment / 100)
			if newCulture < 1 then newCulture = 1 end										-- Ensure Culture is always a positive, non-zero number
			local iCultureDelta = oldCulture - newCulture
			if iCultureDelta < 1 then iCultureDelta = 1 end
			pTargetPlayer:SetJONSCulture(newCulture)										-- Set the new, adjusted culture amount
				
			adjustmentCoefficient = math.random(lowerRange, higherRange)					-- Determine where in the range we set the random adjustment
			local oldGAPoints = pTargetPlayer:GetGoldenAgeProgressMeter()					-- Get the existing Culture amount
			local newGAPoints = math.ceil( oldGAPoints * (adjustmentCoefficient/100) )		-- Determine new Culture amounts as the old amount * (adjustment / 100)
			if newGAPoints < 1 then newGAPoints = 1 end										-- Ensure Culture is always a positive, non-zero number
			local iAGDelta = oldGAPoints - newGAPoints 
			if iAGDelta < 1 then iAGDelta = 1 end
			pTargetPlayer:SetGoldenAgeProgressMeter(newGAPoints)							-- Set the new, adjusted culture amount
	
			NotificationStringFading = "Spreading " .. missionType ..  " reduced " .. targetAdjective .. " progress!"
			NotificationStringLong = "The " .. missionType .. " spread by your Covert Operator " .. pUnit:GetName() .. " has reduced " .. targetAdjective .. " [ICON_CULTURE] Culture progress by " .. iCultureDelta .. " and [ICON_GOLDEN_AGE] Golden Age progress by " .. iAGDelta .."!"				
		else																				-- Poison mission failed; only notification actions are taken
			local missionType = "Rumors"
			if missionNumber == SPREAD_PROPAGANDA then missionType = "Propaganda" end
			NotificationType = 1
			NotificationStringFading = "An attempt to spread " .. missionType .. " near " .. pTargetCity:GetName() .. " failed."  
			NotificationStringLong = "An attempt to spread " .. missionType .. " near " .. pTargetCity:GetName() .. " failed."		
		end

	-- ======================		Mission: Pillage Improvement		===================
	elseif missionNumber == PILLAGE_IMPROVEMENT then	
		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		local improvementName = GetImprovementString(pPlot) or "Improvement"						
		if MissionResult == YES then														-- Pillage mission succeeded	
			pPlot:SetImprovementPillaged(true)
			NotificationStringFading = "Terrain improvement pillaged!"
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " has pillaged the " .. improvementName .. " at his location!"				
		else																				-- Poison mission failed; only notification actions are taken
			local missionType = "Rumors"
			if missionNumber == SPREAD_PROPAGANDA then missionType = "Propaganda" end
			NotificationType = 1
			NotificationStringFading = "Attempt to pillage " .. improvementName .. " near your Covert Operator " .. pUnit:GetName() .. " has failed."  
			NotificationStringLong = "Attempt to pillage the " .. improvementName .. " near your Covert Operator " .. pUnit:GetName() .. " has failed." 		
		end

	-- ======================		Mission: Sabotage Production		===================
	elseif missionNumber == SABOTAGE_PRODUCTION then
	
		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY = pTargetCity:GetY()
				
		if MissionResult == YES then														-- Sabotage Production mission succeeded				
			local buildingProduced =pTargetCity:GetProductionBuilding()
			local projectProduced = pTargetCity:GetProductionProject()
			local unitProduced = pTargetCity:GetProductionUnit()
			
			if buildingProduced ~= -1 then													-- City is producing a building; buildingProduced = building type
				local hammersInvested = pTargetCity:GetProduction()							-- How much production already completed?
				local newHammers															-- How many hammers to remove?			
				local buildingType = Locale.ConvertTextKey(GameInfo.Buildings[buildingProduced].Description) or "Building"		-- Building description

				if GameInfo.BuildingClasses[buildingProduced].MaxGlobalInstances ~= 0 then		-- World Wonder
					newHammers = math.ceil( hammersInvested * (math.random(10, 33) / 100) )	-- How many hammers to remove?
				elseif GameInfo.BuildingClasses[buildingProduced].MaxPlayerInstances ~= 0 then	-- National Wonder
					newHammers = math.ceil( hammersInvested * (math.random(25, 50) / 100) )	-- How many hammers to remove?
				else																				-- Normal building
					newHammers = math.ceil( hammersInvested * (math.random(50, 90) / 100) )	-- How many hammers to remove?
				end
				pTargetCity:ChangeProduction(-newHammers)									-- Remove the hammers 

				NotificationStringFading = buildingType .. " production sabotaged in " .. pTargetCity:GetName() .. "."
				NotificationStringLong = buildingType .. " production sabotaged, by destroying " .. newHammers .. " hammers of progress in " .. pTargetCity:GetName() .. "."

			elseif projectProduced ~= -1 then												-- City is producing a project; projectProduced = project type
				local hammersInvested = pTargetCity:GetProduction()							-- How much production already completed?
				local newHammers = math.ceil( hammersInvested * (math.random(25, 50) / 100) )	-- How many hammers to remove?
				local projectType = Locale.ConvertTextKey(GameInfo.Projects[projectProduced].Description) or "Project"
				
				pTargetCity:ChangeProduction(-newHammers)									-- Remove the hammers 

				NotificationStringFading = projectType .. " production sabotaged in " .. pTargetCity:GetName() .. "."
				NotificationStringLong = projectType .. " production sabotaged, by destroying " .. newHammers .. " hammers of progress in " .. pTargetCity:GetName() .. "."


			elseif unitProduced ~= -1 then													-- City is producing a unit; unitProduced = unit type
				local unitType = Locale.ConvertTextKey(GameInfo.Units[unitProduced].Description) or "Unit"
				
				pTargetCity:SetProduction(1)												-- Set production to 1
				NotificationStringFading = unitType .. " production sabotaged in " .. pTargetCity:GetName() .. "."
				NotificationStringLong = unitType .. " production sabotaged in " .. pTargetCity:GetName() .. "."
			else																			-- City producing something that cannot be sabotaged
				NotificationStringFading = "There is nothing that can be sabotaged in " .. pTargetCity:GetName() .. "."
				NotificationStringLong = " There is nothing that can be sabotaged in " .. pTargetCity:GetName() .. "."
				tMissionResultData[2] = NotificationStringFading
				tMissionResultData[3] = NotificationStringLong
				tMissionResultData[4] = NotificationX
				tMissionResultData[5] = NotificationY
				tMissionResultData[6] = MissionResult
				return																		-- If no valid target, exit early; different result from failing to sabotage...
			end			
		else																				-- Poison mission failed; only notification actions are taken			
			NotificationType = 1
			NotificationStringFading = "Attempt to sabotage production has failed."  
			NotificationStringLong = "Attempt to sabotage production in " .. pTargetCity:GetName() .. " has failed." 		
		end

	-- ======================		Mission: Steal Gold		===================
	elseif missionNumber == STEAL_GOLD then
	
		--local targetPlayer = Players[pTargetCity:GetOwner()]								-- Returns player object for owner of target city   
		local upperRange = 10																-- Default max gold to steal is 10 (or 10%)
		if g_ValidMissions[STEAL_GOLD][3] == YES then										-- City is capital, so higher amount of gold is possible to steal
			upperRange = 25																	-- For the captial the upper max to steal is 25% of all a civ's gold
		end

		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY = pTargetCity:GetY()
		
		if MissionResult == YES then														-- Steal Gold mission succeeded			
			local targetGold = pTargetPlayer:GetGold()										-- Get total gold target civ has on hand
			local stolenAmount = math.ceil( targetGold * (math.random(1, upperRange) / 100) )	-- Determines how much gold was stolen, based on if city is capital or not
			targetGold = targetGold - stolenAmount											-- Determine new gold amount
			if stolenAmount < 1 then stolenAmount = 1 end									-- Ensure it's now lower than 1 gold piece
			pTargetPlayer:SetGold(targetGold)												-- Set gold amount at new level
			pPlayer:ChangeGold(stolenAmount)												-- Adds the stolen gold amount to your treasury

			NotificationStringFading = stolenAmount .. " [ICON_GOLD] Gold stolen from " .. pTargetCity:GetName() .. "."
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " stole " .. stolenAmount .. " [ICON_GOLD] Gold from " .. pTargetCity:GetName() .. "."

		else
			NotificationType = 1
			NotificationStringFading = "Attempt to steal gold has failed."  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " failed in his attempt to steal gold from " .. pTargetCity:GetName() .. "." 	
		end

	-- ======================		Mission: Spread Plague		===================
	elseif missionNumber == SPREAD_PLAGUE then
			
		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY = pTargetCity:GetY()
		--local currentEra = pPlayer:GetCurrentEra()											-- Plague effectiveness tied to ATTACKING player era, not target
		
		if MissionResult == YES then														-- Spread Plague mission succeeded			
			local popLoss = math.random(1, currentEra + 1)									-- Get population to be lost, influenced by Era (higher losses in later Eras)
			local targetCityPop = math.floor(pTargetCity:GetPopulation())					-- Get current target city pop			
			local tradePartnerImpact = NO													-- Tracks if any trade partner cities were hit with plague as well
			if popLoss >= targetCityPop then popLoss = targetCityPop - 1 end				-- Prevent pop from falling below 1			
			local newPop = targetCityPop - popLoss											-- Determine new poplation for target city			
			pTargetCity:SetPopulation(newPop, true)											-- Set target city's pop to the new (lower) number
			

			local targetTrade = pTargetPlayer:GetTradeRoutes()								-- Check if trade partners hit with plague
			for i,v in ipairs(targetTrade) do												-- Scroll through all trade routes for the target player
				if v.FromCity:GetID() == pTargetCity:GetID() then							-- if the trade route originates in target city, then...
					if math.random(1, 100) < 50 then										-- 50:50 chance the city will be hit or not
						if v.ToCity:GetPopulation() >= 2 then								-- if impacted city pop is 2 or more, reduce population, set impact flag to yes
							v.ToCity:ChangePopulation(-1, true)								-- Actually reduce the pop
							tradePartnerImpact = YES										-- Set impact flag on
						end
					end					
				end
				if v.ToCity:GetID() == pTargetCity:GetID() then								-- if the trade route terminates in target city, then...
					if math.random(1, 100) < 50 then										-- 50:50 chance the city will be hit or not
						if v.FromCity:GetPopulation() >= 2 then								-- if impacted city pop is 2 or more, reduce population, set impact flag to yes
							v.FromCity:ChangePopulation(-1, true)							-- Actually reduce the pop
							tradePartnerImpact = YES										-- Set impact flag on
						end
					end					
				end
			end
			
			NotificationStringFading = "Plague has struck the city of " .. pTargetCity:GetName() .. "!"  
			NotificationStringLong = "Plague has struck the city of " .. pTargetCity:GetName() .. "!  The city lost " .. popLoss .. " population."

			if tradePartnerImpact == YES then
				NotificationStringLong = NotificationStringLong .. " The contagion has spread to trading partners of this city!"
			end
		else																				-- Poison mission failed; only notification actions are taken
			NotificationType = 1
			NotificationStringFading = "An attempt to spread plague in the city of " .. pTargetCity:GetName() .. " failed."  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " failed in his attempt to spread plague to " .. pTargetCity:GetName() .. "."
		end

	-- ======================		Mission: Destroy (normal) Building		===================
	elseif missionNumber == DESTROY_BUILDING then
	
		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY =pTargetCity:GetY()

		local destroyedBuildingType	= -1													-- Will hold the random normal building type in a city to by destroyed

		if MissionResult == YES then														-- Destroy Building mission succeeded				
			destroyedBuildingType = GetRandomBuilding(pTargetCity)							-- Returns a random building in city instance sent as argument; non-national and non-world wonder buildings returned	
						
			if destroyedBuildingType ~= -1 then												-- If ~= -1 then there was a valid normal building found
				local buildingName = Locale.ConvertTextKey(GameInfo.Buildings[destroyedBuildingType].Description) or "Building"		-- Building description

				pTargetCity:SetNumRealBuilding(destroyedBuildingType, 0)					-- Removes building type from city			
				NotificationStringFading = buildingName .. " destroyed in the city of " .. pTargetCity:GetName() .. "!"  
				NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " destroyed a " .. buildingName .. " in the city of " .. pTargetCity:GetName() .. "!"
			end
		end
		
		if MissionResult ~= YES	or destroyedBuildingType == -1 then							-- Destroy Building failed or no normal buildings to destroy; only notification actions are taken
			NotificationType = 1
			NotificationStringFading = "Attempt to destroy a building in " .. pTargetCity:GetName() .. " failed."  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " failed in his attempt to destroy a building in the city of " .. pTargetCity:GetName() .. "."
		end

	-- ======================		Mission: Recruit Partisans		===================
	elseif missionNumber == RECRUIT_PARTISANS then

		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY = pTargetCity:GetY()	
		local numUnitsSpawned = -1															-- Defaults to a nil/negative result
		local spawnUnitType = -1															-- Defaults to a nil/negative result
	
		if MissionResult == YES then														-- Recruit Partisans mission succeeded	
			--local targetPlayer = Players[pTargetCity:GetOwner()]							-- Returns player object for owner of target city  FIXX
			local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""			-- Target city owner civ adjective
			--local currentEra = pTargetPlayer:GetCurrentEra()								-- Recruit Partisan effectiveness tied to defending player era, not target		
			
			numUnitsSpawned, spawnUnitType = SpawnRebelUnits(pTargetCity, currentEra - 3)  -- Function generates number of units based on CurrentEra-3 near target city
			if numUnitsSpawned > 0 then
				local unitName = Locale.ConvertTextKey(GameInfo.Units[spawnUnitType].Description)	-- Get the spawned unit's description (name)
				local unitDescriptor = "unit"													-- Is it a 'unit' or 'units' depending on how many spawned - makes the grammar better in the Notification
				if numUnitsSpawned > 1 then unitDescriptor = "units" end

				NotificationType = 3															-- "Rebels Apprear" notification
				NotificationStringFading = targetAdjective .. " partisans have rebelled near " .. pTargetCity:GetName() .. "!"  
				NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " raised an army of " .. targetAdjective .. " partisans, consisting of " .. numUnitsSpawned .. " " .. unitName .. " " .. unitDescriptor .. ", near the city of " .. pTargetCity:GetName() .. "!"
			end
		end
		
		if MissionResult ~= YES	or numUnitsSpawned < 1 then
			NotificationType = 1
			NotificationStringFading = "Attempt to raise rebellion in " .. pTargetCity:GetName() .. " failed."  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " failed in his attempt to raise rebellion in the city of " .. pTargetCity:GetName() .. "."
		end

	
	--elseif missionNumber == SPREAD_PROPAGANDA then										-- Processed in the same area as Spread Rumors

	-- ======================		Mission: Cause Resistance		===================
	elseif missionNumber == CAUSE_RESISTANCE then

		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY = pTargetCity:GetY()
		--local targetPlayer = Players[pTargetCity:GetOwner()]								-- Returns player object for owner of target city  FIXX
		local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""				-- Target city owner civ adjective
			
		if MissionResult == YES then														-- Cause Resistance mission succeeded	
			
			local resistanceDuration = math.random(2, 8)									-- Determine the number of turns the city will enter resistance
			local taretCityPop = pTargetCity:GetPopulation()
			if resistanceDuration > taretCityPop then resistanceDuration = taretCityPop end			
			pTargetCity:ChangeResistanceTurns(resistanceDuration)			
						
			NotificationStringFading = "The city of " .. pTargetCity:GetName() .. " is experiencing resistance!"  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " has created resistance in the " .. targetAdjective .. " city of " .. pTargetCity:GetName() .. "!"
		else		
			NotificationType = 1
			NotificationStringFading = "Attempt to create resistance in " .. pTargetCity:GetName() .. " failed."  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " failed in his attempt create resistance in the " .. targetAdjective .. " city of " .. pTargetCity:GetName() .. "."
		end

	-- ======================		Mission: Cause Anarchy		===================
	elseif missionNumber == CAUSE_ANARCHY then

		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)		-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY = pTargetCity:GetY()
		--local targetPlayer = Players[pTargetCity:GetOwner()]								-- Returns player object for owner of target city  FIXX
		local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""				-- Target city owner civ adjective
			
		if MissionResult == YES then														-- Cause Anarchy mission succeeded				
			local anarchyDuration = math.random(1, 3)										-- Determine the number of turns the target civ will enter anarchy (no gold income, science, or production)
			if pTargetPlayer:IsAnarchy() then												-- If civ is already in anarchy, this will add anarchyDuration turns
				pTargetPlayer:ChangeAnarchyNumTurns(anarchyDuration)
			else																			-- If not already in anarchy, will set anarchy on for anarchyDuration turns
				pTargetPlayer:SetAnarchyNumTurns(anarchyDuration)
			end			
						
			NotificationStringFading = "The " .. targetAdjective .. " civilization is experiencing anarchy!"  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " has created anarchy throughout the " .. targetAdjective .. " civilization!"
		else		
			NotificationType = 1
			NotificationStringFading = "Attempt to create anarchy in the " .. targetAdjective .. " civilization failed."  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " failed in his attempt create anarchy in the " .. targetAdjective .. " civilization."
		end

	-- ======================		Mission: Steal City		===================
	elseif missionNumber == STEAL_CITY then
	
		MissionResult = GetMissionOutcome(missionNumber, iAttackingOperatorLevel, pTargetCity)	-- Get mission results
		NotificationX = pTargetCity:GetX()													-- Notification for this message moves focus to attacked city
		NotificationY = pTargetCity:GetY()
		local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""				-- Target city owner civ adjective
		local targetCurrentEra = pTargetPlayer:GetCurrentEra()								-- Recruit Partisan effectiveness tied to defending player era, not target		
			
		if MissionResult == YES then														-- Steal City mission succeeded	
					
			local numUnitsSpawned, spawnUnitType = SpawnRebelUnits(pTargetCity, targetCurrentEra - 3)  -- Function generates number of units based on CurrentEra-3 near target cit
			pPlayer:AcquireCity(pTargetCity, true, false)									-- pPlayer acquires target city, 2nd arg=Conquest, 3rd arg=Trade)
						
			NotificationStringFading = "The city of " .. pTargetCity:GetName() .. " has decided to join your empire!"  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " has inspired the citizens of the " .. targetAdjective .. " civilization's city of " .. pTargetCity:GetName() .. " to join your empire!"
			if numUnitsSpawned > 0 then
				local unitName = Locale.ConvertTextKey(GameInfo.Units[spawnUnitType].Description)	-- Get the spawned unit's description (name)
				NotificationStringLong = NotificationStringLong .. " Rebel " .. unitName .. " units have been seen near the city!"
			end
		else		
			NotificationType = 1
			NotificationStringFading = "Attempt to steal the city of " .. pTargetCity:GetName() .. " failed."  
			NotificationStringLong = "Your Covert Operator " .. pUnit:GetName() .. " failed in his attempt to steal the " .. targetAdjective .. " city of " .. pTargetCity:GetName() .. "."
		end
	-- ======================		Mission: Abort Mission		===================
	elseif missionNumber == ABORT_MISSION then												-- Abort Mission - always available selection
		--print("Mission aborted...")
		return																				-- Exit early - no notifications since nothing happened...
	else
		print("Big error here ----  couldn't find a valid mission....")
		return
	end
	
	tMissionResultData[1] = NotificationType												-- Built return table data
	tMissionResultData[2] = NotificationStringFading
	tMissionResultData[3] = NotificationStringLong
	tMissionResultData[4] = NotificationX
	tMissionResultData[5] = NotificationY
	tMissionResultData[6] = MissionResult
	--tMissionResultData[7] = SurviveResult
	--tMissionResultData[8] = IncidentResult

end


-- ========================================================================================
--	Process Human Player Covert Missions:  Only arrive here by way of Mission Menu Popup
-- ========================================================================================

function ProcessMission (missionNumber)														-- Function determines mission results - FOR THE HUMAN PLAYER!  
	--print("==> Inside processmission.  Mission: " .. g_MissionList[missionNumber][1] .. "    Mission Number: " .. missionNumber)
	if missionNumber == ABORT_MISSION then return end										-- User selected "Abort Mission" - no need for further processing

	local pPlayer = Players[g_CurrentPlayer]												-- Get basic player, unit, plot data for processing; pPlayer is the "attacking" player
	local pUnit = UI.GetHeadSelectedUnit()													-- Default Covert Operator is the currently "selected" unit; pUnit is the "attacking" unit
	local currentEra = pPlayer:GetCurrentEra()												-- Get Active (human) player's current level

	local pTargetPlayer = Players[g_ValidMissions[missionNumber][4]]						-- [4] is the target player ID
	local pTargetCity = g_ValidMissions[missionNumber][2]									-- [2] is the target city instance, which is null for mission PILLAGE_IMPROVEMENT
		
	-- Mission results:		    Type	FadeStr	LongStr		X		Y		Msn		Survive		Incident  -- default values for mission results data table
	local tMissionResultData = { 1,		"",		"",			-1,		-1,		1,		1,			1 }
	
	GetMissionResult(tMissionResultData, missionNumber, pPlayer, pUnit, currentEra, pTargetPlayer, pTargetCity)
		
	GetInternationalResponses(tMissionResultData, missionNumber, pPlayer, pUnit, pTargetPlayer, pTargetCity)
			
	GetOperatorDestiny(tMissionResultData, missionNumber, pPlayer, pUnit, pTargetCity)
	
	SendNotification(tMissionResultData[1], tMissionResultData[3], tMissionResultData[2], pUnit, tMissionResultData[4], tMissionResultData[5])
		
end
