-- Process_International_Response.lua
-- Author: sman
-- DateCreated: 01/08/18 9:19:38 AM
--------------------------------------------------------------

--include("00_Globals.lua")

-- =================================================================================================================================
-- Reponse global data

RESPONSE_MUTED = 1																			-- Incidence Severity Enumerations
RESPONSE_MINOR = 2
RESPONSE_DENOUNCE = 3
RESPONSE_DENOUNCE_WITH_ALLIES = 4
RESPONSE_WAR = 5
RESPONSE_WAR_WITH_ALLIES = 6
RESPONSE_WAR_PERMANENT = 7
RESPONSE_WAR_GLOBAL = 8

g_tIncidenceResponse = {  [RESPONSE_MUTED] = 1, [RESPONSE_MINOR] = 20, [RESPONSE_DENOUNCE] = 40, [RESPONSE_DENOUNCE_WITH_ALLIES] = 55, [RESPONSE_WAR] = 70, 
							[RESPONSE_WAR_WITH_ALLIES] = 80, [RESPONSE_WAR_PERMANENT] = 90, [RESPONSE_WAR_GLOBAL] = 95	}

g_iDomesticResponseProbability = 50															-- Percent chance of domestic resistance to Covert Ops; adjusted later by excess happiness
g_iAlliedReponseProbability = 66															-- Percent chance allies will denounce or declare war as a response to an incident
g_iGlobalWarResponseProbability = 50														-- Percent chance any civ will declare war as a result of an incident when RESPONSE_WAR_GLOBAL is selected


-- =================================================================================================================================
-- Functions

function GetIncidentOutcome(iMissionNumber, iAttackingOperatorLevel, pPlayer, pTargetPlayer)		-- Determines what happened during the covert mission (successful? operator survive? incident?)
	local iIncidentResult = NO																-- Default result is there was no incident

	--  Adjust results based on level (competence) of Operator	
	local iAdjustmentByOperatorLevel = math.floor((iAttackingOperatorLevel - 1) * 7)		-- Level 1 gets no bonus, Level 4 gets 21% better chance of positive result

	-- Get incident results; if target player and active player already at war, there will be no incident
	local pTargetTeam = Teams[pTargetPlayer:GetTeam()]										-- Get team instance for target civ
	local iAttackingTeamID = pPlayer:GetTeam()												-- Get Team ID for the attacking civ
	
	if not pTargetTeam:IsAtWar(iAttackingTeamID) then										-- Only test for incident if both civs at peace; otherwise default stands (NO)
		if math.random(1, 100) < (g_MissionList[iMissionNumber][4] + iAdjustmentByOperatorLevel) then	-- Did mission cause Incident?  Based on chance for specific mission [4]
			iIncidentResult = YES															-- Change default return value 
		end
	end
	--print("====> Results.  incident: " .. IncidentResult)
	return iIncidentResult
end


function CheckDomesticResponse(pPlayer)
	local iResponseFloor = 90
	local iReponse = NO																		-- Default return to "NO" (domestic response)
	local iResponseChance = g_iDomesticResponseProbability - pPlayer:GetExcessHappiness()	-- Lower the chance of response by excess happiness
	if iResponseChance < iResponseFloor then iResponseChance = iResponseFloor end			-- Ensure at least xx% (iResponseFloor) chance of response
	if math.random(1, 100) < iResponseChance then											-- Check random result against chance of response
		iReponse = YES																		-- If random number is under that chance, set return to YES
	end
	return iReponse																			-- Return result
end


-- ==========================================================================================================
-- Process Incident Results

function GetInternationalResponses(tMissionResultData, missionNumber, pPlayer, pUnit, pTargetPlayer, pTargetCity)

	local IncidentResult = NO		
	local NotificationStringFading = ""														-- Clear notification strings for later appending
	local NotificationStringLong = ""
	
	local iAttackingOperatorLevel = 1														-- Initial value for the level of the operator launching an attack: should be changed for either AI or human player with other values
	if not pPlayer:IsHuman() then 	
		iAttackingOperatorLevel = g_MissionList[missionNumber][6]							-- Sets the value to the Level value of the mission itself
	else
		if pUnit then																		-- iAttackingOperatorLevel defaults to 1 if pUnit is nil
			iAttackingOperatorLevel = GameInfo.Units[pUnit:GetUnitType()].IsCovertOperatorUnit or 1	-- Attacking unit CO level - should default to level 1 if pUnit is nil (for AI attacks)		
		end
	end

	IncidentResult = GetIncidentOutcome(missionNumber, iAttackingOperatorLevel, pPlayer, pTargetPlayer)

	if IncidentResult == YES then															-- If YES, then there is an international incident to process
		local iResponseSeverity = math.random(1, 100)										-- Just how severe is the aftermath?						

		-- Adjust the severity based on mission level.  The higher level the mission, the higher the chance for a more serious response
		-- For L1 missions, no change; for L2 missions, add 3-11% to the response severity; for L3, add 6-22%; for L4, add 9-33%
		local iSeverityAdjustment = math.random((g_MissionList[missionNumber][6] - 1) * 3, (g_MissionList[missionNumber][6] - 1) * 11)
		iResponseSeverity = iResponseSeverity + iSeverityAdjustment

		local iIndex = #g_tIncidenceResponse												-- Set the index to the last element in the "incidence response" table	
		for idx = 1, #g_tIncidenceResponse - 1 do											-- Move through the incidence response table from frist element until next-to-last
			if (iResponseSeverity >= g_tIncidenceResponse[idx]) and (iResponseSeverity < g_tIncidenceResponse[idx+1]) then		-- If the score is between this element and the next, then
				iIndex = idx																-- Set the return index to the loop index value
				break																		-- Terminate the loop early, we've found what we're looking for
			end
		end			
		--print("Inside responses:  severity: " .. iResponseSeverity .. "    iIndex: " .. iIndex)

		-- ======================		Response: Muted ===================
		if iIndex == RESPONSE_MUTED then													-- Minor notification
			NotificationStringLong = NotificationStringLong .. "There has been a very muted international response to this incident."

		-- ======================		Response: Minor ===================
		elseif iIndex == RESPONSE_MINOR then												-- Minor notification, with possibility of brief resistance in the capital
			NotificationStringLong = NotificationStringLong .. "There has been only a minor international murmur about this incident."		
		
		-- ======================		Response: Denounce ===================
		elseif iIndex == RESPONSE_DENOUNCE then												-- Issue denunciation, with possibility of brief resistance in the capital (processed below)
			if (pTargetPlayer:GetID() < GameDefines.MAX_MAJOR_CIVS-1)  and					-- Only major civs can denounce; if we get here and target is City State, no "real" response will occur or be included in Notification string
					( not pTargetPlayer:IsHuman() )  then					
				NotificationStringLong = NotificationStringLong .. pTargetPlayer:GetCivilizationShortDescription() .. " has denounced " .. pPlayer:GetCivilizationShortDescription() .. " !"		
				pTargetPlayer:DoForceDenounce(pPlayer:GetID())								-- Force denunciation of target civ to the active civ		
			else																			-- Target city was City State, so general message, no other action; some possible domestic response further down		
				NotificationStringLong = NotificationStringLong .. "There has been minimal international response about this incident."			
			end
			
		-- ======================		Response: Team Denounce ===================
		elseif iIndex == RESPONSE_DENOUNCE_WITH_ALLIES then									-- Possible denunciation by target civ and all its DOF's, with possibility of brief resistance in the capital
																							-- Only non-City States can denounce!			
			local sTempString = "These Civilizations have denounced " .. pPlayer:GetCivilizationShortDescription() .. ":  "			
			local iDenouncingCivs = 0
			local iTargetPlayerID = pTargetPlayer:GetID()									-- Get the target civ's team ID
			
			for idx = 1, GameDefines.MAX_MAJOR_CIVS-1, 1 do									-- Loop through all major AI players (skip human = 0)				
				local pTempPlayer = Players[idx]											-- Get the loop player's instance
				local pTempTeam = Teams[pTempPlayer:GetTeam()]								-- Get team pointer for loop player
				
				if pTempPlayer:IsAlive() and												-- Only check civs still in the game
						 pTempTeam:IsHasMet(pPlayer:GetTeam())	and							-- Has this civ met the attacking player?  GetTeam returns Team ID
						( pTempPlayer:IsDoF(iTargetPlayerID) or iTargetPlayerID == idx ) then		-- Civ has DOF with target civ or is the target civ
					
					if math.random(1, 100) < g_iAlliedReponseProbability then				-- Random # below threshold (66% chance), allied civ will respond!
						pTempPlayer:DoForceDenounce(pPlayer:GetID())						-- Force the denunciation
						iDenouncingCivs = iDenouncingCivs + 1								-- Bump the denouncing civ counter
						sTempString = sTempString.. pTempPlayer:GetCivilizationShortDescription() .. "   "	-- Add the civ to the running string
					end
				end
			end
			if iDenouncingCivs == 0 then													-- If no one denounced, empty the string 			 
				sTempString = "" 
			else
				sTempString = sTempString .. "[NEWLINE]"
			end							
			NotificationStringLong = NotificationStringLong .. sTempString			
					
		-- ======================		Response: Target civ declares either War or Permanent War ===================
		elseif (iIndex == RESPONSE_WAR) or (iIndex == RESPONSE_WAR_PERMANENT) then			-- Target civ declares war, permanently in extreme cases; with possibility of brief resistance in the capital
			local sDuration = ""
			local pTargetTeam = Teams[pTargetPlayer:GetTeam()]								-- Get team pointer for target player
			local pPlayerTeam = Teams[pPlayer:GetTeam()]									-- Get team pointer for attacking player
			pTargetTeam:DeclareWar(pPlayerTeam:GetID())										-- Force DOW
			if iIndex == RESPONSE_WAR_PERMANENT then										-- If this is a permanent DOW, then
				pTargetTeam:SetPermanentWarPeace(pPlayerTeam:GetID(), true)					-- Set DOW to permanent
				sDuration = "PERMANENT "													-- Adjust duration description
			end
			NotificationStringLong = NotificationStringLong .. pTargetPlayer:GetCivilizationShortDescription() .. " has declared " .. sDuration .. "war on " .. pPlayer:GetCivilizationShortDescription() .. "!"		

		-- ======================		Response: Team War ===================
		elseif iIndex == RESPONSE_WAR_WITH_ALLIES then										-- Possible DOW's by target civ and all its DOF's, with possibility of brief resistance in the capital		
			local pPlayerTeam = Teams[pPlayer:GetTeam()]									-- Get team pointer for attacking player
			local sTempString = "These Civilizations have declared war on " .. pPlayer:GetCivilizationShortDescription() .. ":  "			
			local iDenouncingCivs = 0
			local iTargetPlayerID = pTargetPlayer:GetID()									-- Get the target civ's team ID
			local pPlayerTeam = Teams[pPlayer:GetTeam()]									-- Get team pointer for attacking player

			for idx = 1, GameDefines.MAX_MAJOR_CIVS-1, 1 do									-- Loop through all major AI players (skip human = 0)				
				local pTempPlayer = Players[idx]											-- Get the loop team's team ID
				local pTempTeam = Teams[pTempPlayer:GetTeam()]								-- Get team pointer for target player

				if pTempPlayer:IsAlive() and												-- Only test for civs still alive
						pTempTeam:IsHasMet(pPlayer:GetTeam()) and							-- Has this civ met the attacking player?  GetTeam returns Team ID
						( pTempPlayer:IsDoF(iTargetPlayerID)  or  iTargetPlayerID == idx ) then		-- Civ DOF with target civ or is the target civ

					if math.random(1, 100) < g_iAlliedReponseProbability then				-- Random # below threshold, allied civ will respond!						
						pTempTeam:DeclareWar(pPlayerTeam:GetID())							-- Force DOW											
						iDenouncingCivs = iDenouncingCivs + 1								-- Bump the denouncing civ counter
						sTempString = sTempString.. pTempPlayer:GetCivilizationShortDescription() .. "   "	-- Append the civ to the running string
					end
				end
			end
			if iDenouncingCivs == 0 then													-- If no one denounced, empty the string 			 
				sTempString = "" 
			else
				sTempString = sTempString .. "[NEWLINE]"
			end							
			NotificationStringLong = NotificationStringLong .. sTempString

		-- ======================		Response: Permanent War with target ===================
		--elseif iIndex == RESPONSE_WAR_PERMANENT then

		-- ======================		Response: Global War ===================
		elseif iIndex == RESPONSE_WAR_GLOBAL then											-- Possible DOW's by all known civs, with possibility of brief resistance in the capital
																							-- Note: different civs have different probabilities to DOW: DOF's have highest chance, Civs who have denounced Active civ have next highest, normal civs have lowest chance
			local pPlayerTeam = Teams[pPlayer:GetTeam()]									-- Get team pointer for atacking player
			local sTempString = "These Civilizations have declared war on " .. pPlayer:GetCivilizationShortDescription() .. ":  "			
			local iDenouncingCivs = 0
			local iTargetPlayerID = pTargetPlayer:GetID()									-- Get the target civ's team ID
			
			for idx = 1, GameDefines.MAX_MAJOR_CIVS-1, 1 do									-- Loop through all major AI players (skip human = 0)				
				local iDOW = NO
				local pTempPlayer = Players[idx]											-- Get the loop players instance
				local pTempTeam = Teams[pTempPlayer:GetTeam()]								-- Get team pointer for loop player

				if (pTempPlayer:IsAlive()) and (pTempTeam:IsHasMet(pPlayer:GetTeam()) ) then	-- If civ is alive and has met Active player, test for relationship/DOW chance
								
					if pTempPlayer:IsDoF(iTargetPlayerID) or (iTargetPlayerID == idx) then	-- Loop Civ has declared DOF with target civ or is target civ
						if math.random(1, 100) < g_iAlliedReponseProbability then			-- Random # below threshold, civ will respond!
							iDOW = YES
						end
				
					elseif pTempPlayer:IsDenouncingPlayer(pPlayer:GetID()) then			-- Has this player denounced Attacking player?  if so, then use delta between allies and all others' probabilities - split the difference
						if math.random(1, 100) < (g_iAlliedReponseProbability - g_iGlobalWarResponseProbability) then		-- Random # below threshold, civ will respond!
							iDOW = YES
						end
				
					else																	-- All other players use this probability
						if math.random(1, 100) < g_iGlobalWarResponseProbability then		-- Random # below threshold, civ will respond!
							iDOW = YES
						end
					end
					if iDOW == YES then														-- If any of the tests worked, start DOW						
						pTempTeam:DeclareWar(pPlayer:GetTeam())								-- Force DOW											
						iDenouncingCivs = iDenouncingCivs + 1								-- Bump the denouncing civ counter
						sTempString = sTempString.. pTempPlayer:GetCivilizationShortDescription() .. "   "	-- Add the civ to the running string
					end
				end
			end
			if iDenouncingCivs == 0 then													-- If no one denounced, empty the string 			 
				sTempString = "" 
			end							
			NotificationStringLong = NotificationStringLong .. sTempString

		else																				-- Should never reach here
			print("End of Incidence Response without a valid iIndex, whose value is: " .. iIndex)
			NotificationStringLong = NotificationStringLong .. "This incident has caused an extensive international backlash!"	-- Indeterminate response, so issue a "safe" response
		end

		-- ==================================================================================================================================
		--		Domestic Turmoil?  
		local iDomesticTurmoilTurns = iIndex - 2											-- Potential "domestic turmoil" turns based on severity of international response (3/Denounce or higher)
		if iDomesticTurmoilTurns < 1 then iDomesticTurmoilTurns = 0 end						-- If less than 1, there is no domestic response

		if iDomesticTurmoilTurns > 0 then													-- If we have a domestic response, then...
			local iDomesticDisapproval = CheckDomesticResponse(pPlayer)						-- Determine if people are upset about incident; returns YES or NO
			if iDomesticDisapproval == YES then												-- If so, then
				local pCapitalCity = pPlayer:GetCapitalCity()								-- Get the capital city instance
				local iResistanceDuration = math.random(1, iDomesticTurmoilTurns)			-- Set duration of Resistance for 1..iDomesticTurmoilTurns  (should be 6 max)
				pCapitalCity:ChangeResistanceTurns(iResistanceDuration)						-- These resistance turns are cumulative!  This action adds turns to previous Resistance
				NotificationStringLong = NotificationStringLong .. "The citizens of the " .. pPlayer:GetCivilizationAdjective() .. " capital are protesting their leadership (" .. iResistanceDuration .. " turns)."
			end	
		end
		
	else																					-- Mission response was "NO" 
		-- DO WE NEED An ELSE response?????????  "NO RESULT" = no report.  If at war, never see this so will always come to this else - to say what?  No response?  nothing happened?
		-- Currently, leave no message on this event.
	end
				
	tMissionResultData[2] = tMissionResultData[2] .. "  " .. NotificationStringFading	-- Built return table data
	tMissionResultData[3] = tMissionResultData[3] .. "  " .. NotificationStringLong
	tMissionResultData[8] = IncidentResult
end