-- Process_Recon_Results.lua
-- Author: sman
-- DateCreated: 12/27/17 12:28:51 PM
--------------------------------------------------------------

--include ("00_Globals.lua")


-- ===============================================================================================
--		Globals

g_iLocalReportChance = 67																	-- Percent chance the report will use the g_tCityReconPercentages table
																							-- if random 1..100 is greater than this chance, a g_tCivReconPercentages report will be made 

g_tCityReconPercentages = { [1] = { "Local Unit Count",				 1},					-- Function will generate a random number from 1 to 100
							[2] = { "Local Gold Production",		11},					-- The number generated will determine which of these reports will be created
							[3] = { "Local Science Production",		21},	
							[4] = { "Local Culture Production",		31},	
							[5] = { "Local Faith Production",		41},	
							[6] = { "Local Happiness",				51},	
							[7] = { "Local Number of Buildings",	60},	
							[8] = { "Local Number of Wonders",		65},	
							[9] = { "List of World Wonders",		70},
							[10] = { "List of National Wonders",	75},
							[11] = { "List of Buildings",			80},	
							[12] = { "Current Production",			85} }	
							

g_tCivReconPercentages = {	[1] = { "Current Era",						 1},				-- Function will generate a random number from 1 to 100
							[2] = { "Technology Being Researched",		 4},				-- The number generated will determine which of these reports will be created
							[3] = { "Number of Technologies Researched", 9},	
							[4] = { "Technology Progress",				13},	
							[5] = { "Policy Progress",					17},	
							[6] = { "Golden Age Progress",				21},	
							[7] = { "Wonder Being Built",				24},
							[8] = { "Wonder Progress",					28},	
							[9] = { "Number of Wonders Already Built",	32},	
							[10] = { "Number of Policies Unlocked",		36},	

							[11] = { "Global Gold Production",			40},	
							[12] = { "Global Science Production",		44},	
							[13] = { "Global Culture Production",		52},	
							[14] = { "Global Faith Production",			56},	
							[15] = { "Global Happiness",				60},	
							
							[16] = { "Military Might",					64},
							[17] = { "Total Unit Count",				67},
							[18] = { "Total Unit Out of Supply Count",	70},
							[19] = { "Total Nuclear Unit Count",		73},
							[20] = { "Total Spy Count",					76},
							[21] = { "Spy Locations",					79},		
							
							[22] = { "Building Count",					82},	
							[23] = { "Building Maintenance Cost",		85},	

							[24] = { "Score (Land Component)",			88},	
							[25] = { "Score (Cities Component)",		90},	
							[26] = { "Score (Population Component)",	92},	
							[27] = { "Score (Wonders Component)",		94},	
							[28] = { "Score (Tech Component)",			96}, 
							[29] = { "Total Building Class Count",		97} }

g_tReliability_Adjective =	{ "Rumor", "Possible", "Probable", "Almost Certain"}			-- Describes the quality of the report

g_tReliability_Coefficient = { 25, 20, 15, 10 }												-- Report data will be + or - amount from actual number	

g_tReliability_Probability = { 1, 15, 51, 87 }												-- Percentage start for this quality of report (between 1..100)


-- ===============================================================================================
--		Support Functions

function GetIndexByPercentage(iScope, iPercentage)											-- Returns the correct index for "reconpercentages" tables, based on iPercentage		
	local tTempTable = g_tCityReconPercentages												-- Default table is the local one
	if iScope > g_iLocalReportChance then tTempTable = g_tCivReconPercentages end			-- If the iScope is high enough, change the table to the global one
	local iTableSize = #tTempTable															-- Determine working table size
	local iIndex = iTableSize																-- Default the index to return as the last element in the talbe
	for idx = 1, iTableSize - 1 do															-- Move through the working table from frist element until next-to-last
		if (iPercentage >= tTempTable[idx][2]) and (iPercentage < tTempTable[idx+1][2]) then		-- If the target percentage is between this element and the next element, then
			iIndex = idx																	-- Set the return index to the loop index value
			break																			-- Terminate the loop, we've found what we're looking for
		end
	end	
	return iIndex																			-- Return the found index
end

function GetIndexByReliability(iReliabilityScore)											-- Returns the correct index used by the Reliability tables based on the reliability score
	if not iReliabilityScore then iReliabilityScore = 1 end									-- If for some reason we don't have a reliability score, set it to 1, lowest possibility
	local iIndex = #g_tReliability_Probability												-- Set the index to the last element in the "reliability probability" table	
	for idx = 1, #g_tReliability_Probability-1 do											-- Move through the reliability probablity table from frist element until next-to-last
		if (iReliabilityScore >= g_tReliability_Probability[idx]) and (iReliabilityScore < g_tReliability_Probability[idx+1]) then		-- If the score is between this element and the next, then
			iIndex = idx																	-- Set the return index to the loop index value
			break																			-- Terminate the loop early, we've found what we're looking for
		end
	end	
	return iIndex																			-- Return the found index
end

function AdjustCount(iNumber, iReliabiltyIDX, bIsMinusOK)									-- Adjusts "iNumber" by a random +/- amount, based on how "reliable" the report is
	if not iNumber then iNumber = 0 end														-- Ensure iNumber isn't nil; least impacting default value
	if not iReliabiltyIDX then iReliabiltyIDX = 1 end										-- Ensure iReliabilityIDX isn't nil; least impacting default value
	local iNewNumber = 0																	-- Initialize return value to 0
	local iAdjustment =  math.floor(iNumber * (g_tReliability_Coefficient[iReliabiltyIDX]/100))		-- Determine the adjustment number as a percentage of the iNumber, set by the reliability coefficient
	if iAdjustment < 1 then iAdjustment = 1 end												-- If the adjustment is < 1, make it 1; always want some variety possible in these numbers
	iNewNumber = math.random(iNumber - iAdjustment, iNumber + iAdjustment)					-- Adjust the return value randomly from (number-adjustment) to (number+adjustment)
	if (bIsMinusOK == false) and (iNewNumber < 0) then iNewNumber = 0 end					-- If it's NOT OK to have negative numbers, ensure the return value is no lower than 0
	--print("AdjustCount.  Reliability Index: " .. iReliabiltyIDX .. "   Old #: " .. iNumber .. "   New #: " .. iNewNumber)
	return iNewNumber																		-- Return the adjusted number
end

function GenerateGameDate()																	-- Returns a random date string, based on current game turn
	local sGameYear = "Unknown"																-- Default return string is "unk"
	local iCurrentGameYear = Game.GetGameTurnYear() or nil									-- Get game year or nil
	if iCurrentGameYear then																-- If we have a valid game year (not nil), then
		local sYearQualifier = "AD"															-- Default to an "AD" year
		if iCurrentGameYear < 1 then														-- If game year is negative, then
			sYearQualifier = "BC"															-- Change the qualifier to "BC"
		end
		local iReportDate = math.random(1, 28)												-- Generate a random day of month
		local sSuffix = "th"																-- Default suffix for ordinal numbers		
		if (iReportDate == 1) or (iReportDate == 21) then									-- CHange the DOM suffix if needed
			sSuffix = "st"
		elseif (iReportDate == 2) or (iReportDate == 22) then
			sSuffix = "nd"
		elseif (iReportDate == 3) or (iReportDate == 23) then
			sSuffix = "rd"
		end
		local iMonthIdx = math.random(1, 12)												-- Get month, 1..12
		local sMonth = string.sub(g_tMonths[iMonthIdx], 1, 3)								-- Set month string at first 3 letters of month
		if iMonthIdx ~= 5 then																-- If month is not May, then, ...
			sMonth = sMonth .. "."															-- Add a period at the end of the abbreviation
		end
		sGameYear =  sMonth .. " " .. iReportDate .. sSuffix .. ", " ..  math.abs(iCurrentGameYear) .. " " .. sYearQualifier  -- Build date string
	end	
	return sGameYear																		-- Return date string
end

-- ===============================================================================================
--		Main Function -- Determines what report will be returned and the "quality" (accuracy) of the report

function ProcessReconMissionResults(pUnit, pTargetCity)	
	
	local pTargetPlayer = Players[pTargetCity:GetOwner()]									-- Player instance of the civ we're reconning
	local sTargetPlayerShortDescription = pTargetPlayer:GetCivilizationShortDescription()	-- Target player's adjective, i.e. "Greek"
	local sTargetPlayerAdjective = pTargetPlayer:GetCivilizationAdjective()					-- Target player's short description, i.e. "China"
	local pActivePlayerTeam = Teams[Players[Game.GetActivePlayer()]:GetTeam()]				-- Get the team of the active (human) player
	local sActivePlayerShortDescription = Players[Game.GetActivePlayer()]:GetCivilizationShortDescription()		-- Active player's short description
	local sActivePlayerAdjective = Players[Game.GetActivePlayer()]:GetCivilizationAdjective()	-- Active player's adjective

	local sReportingAgentName = pUnit:GetName() or "Unknown Agent"							-- Get unique short name for this unit 
	local sReportDate = GenerateGameDate()													-- Generate random date to include in report, based on game turn

	local sMessageLength = "-----------------------------------------------------------------------------------"	-- Various settings to formate the report
	local sReconHeader = "Intel Report from Agent Near " .. pTargetCity:GetName()
	local sReconReport = sMessageLength .. "[NEWLINE]Reporting Operative:  " .. sReportingAgentName .. "[NEWLINE]"	 
	local sReconReport = sReconReport .. "Signed: " .. sReportDate .. "[NEWLINE]"
	local sReconReport = sReconReport .. sMessageLength .. "[NEWLINE]"
	local sReconSummary = ""
	
	local iUnitLevel = GameInfo.Units[pUnit:GetUnitType()].IsCovertOperatorUnit or 1		-- Get the level of the operator unit
	local iReportScope = math.random(1, 100)												-- Determine the scope of the report (Local or Global/city or civ-wide)
	local iReportTopic = math.random(1, 100)												-- Determine the type of report to generate
	local iReconReliabilityScore = math.random(1, 100)										-- Initial quality scoreof the mission
	iReconReliabilityScore = iReconReliabilityScore + ((iUnitLevel - 1) * 10)				-- Higher level units get better scores, 10% per level after level 1
	if iReconReliabilityScore > 100 then iReconReliabilityScore = 100 end					-- Score must be between 1..100
	local iReliabiltyIDX = GetIndexByReliability(iReconReliabilityScore)					-- Get the correct index based on the reliability score
	local iReconReportIDX = GetIndexByPercentage(iReportScope, iReportTopic)				-- Get the correct index, based on the random topic score

	--iReportScope = 99
	--iReconReportIDX = math.random(22, 29)

	-- =================================================================================	   Process Intelligence Reports
	if iReportScope <= g_iLocalReportChance then											-- Local (city) reports  	
	
		-- ======================		Report:	Local unit count	=========================	
		if iReconReportIDX == 1 then
			local iTargetCityUnits = 0														-- Set default number of units to 0
			local iForeignUnits = 0		
			local pTargetCityPlotX = pTargetCity:GetX()										-- Get the correct X for target city
			local pTargetCityPlotY = pTargetCity:GetY()										-- Get the correct Y for target city

			for iDX = -3, 3 do																-- Search target city x & y  + - 3 tiles
				for iDY = -3, 3 do
					local pTargetPlot = Map.PlotXYWithRangeCheck(pTargetCityPlotX, pTargetCityPlotY, iDX, iDY, 3)		-- Get the plot to examine					
					if pTargetPlot then														-- If plot is valid, then...
						local iNumUnits = pTargetPlot:GetNumUnits()							-- Determine number of units in this plot
						for j = 0, iNumUnits do												-- Cycle through all the units in this plot
							pTempUnit = pTargetPlot:GetUnit(j)								-- Get unit 'j' as a unit instance
							if pTempUnit ~= nil then										-- If valid unit instance...
								if pTempUnit:GetOwner() == pTargetCity:GetOwner() then		-- If unit 'j' is owned by target city owner, bump that count
									iTargetCityUnits = iTargetCityUnits + 1
								else														-- Otherwise, bump the non-city owner count
									iForeignUnits = iForeignUnits + 1
								end
							end
						end
					end
				end
			end
			
			iTargetCityUnits = AdjustCount(iTargetCityUnits, iReliabiltyIDX, false)			-- Adjust report results based on quality of report
			iForeignUnits = AdjustCount(iForeignUnits, iReliabiltyIDX, false)				-- Adjust report results based on quality of report
									
			sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " has " .. iTargetCityUnits .. " of its units nearby[NEWLINE]"
			sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " has " .. iForeignUnits .. " foreign units nearby"
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " has " .. iTargetCityUnits .. " friendly units nearby"

		-- ======================		Report:	Local gold production	=========================
		elseif iReconReportIDX == 2 then			
			local iTargetCityGold = pTargetCity:GetYieldRate(YieldTypes.YIELD_GOLD)			-- Get city's yield of gold per turn
			iTargetCityGold = AdjustCount(iTargetCityGold, iReliabiltyIDX, true)			-- Adjust report results based on quality of report

			sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " generates " .. iTargetCityGold .. " [ICON_GOLD] gold per turn"
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " generates " .. iTargetCityGold .. " gold per turn"
			
		-- ======================		Report:	Local science production	=========================
		elseif iReconReportIDX == 3 then	
			local iTargetCityScience = pTargetCity:GetYieldRate(YieldTypes.YIELD_SCIENCE)	-- Get city's yield of science per turn
			iTargetCityScience = AdjustCount(iTargetCityScience, iReliabiltyIDX, true)		-- Adjust report results based on quality of report

			sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " generates " .. iTargetCityScience .. " [ICON_RESEARCH] science per turn"
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " generates " .. iTargetCityScience .. " science per turn"

		-- ======================		Report:	Local culture production	=========================
		elseif iReconReportIDX == 4 then													
			local iTargetCityCulture = pTargetCity:GetYieldRate(YieldTypes.YIELD_CULTURE)	-- Get city's yield of culture per turn
			iTargetCityCulture = AdjustCount(iTargetCityCulture, iReliabiltyIDX, false)		-- Adjust report results based on quality of report

			sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " generates " .. iTargetCityCulture .. " [ICON_CULTURE] culture per turn"
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " generates " .. iTargetCityCulture .. " science per turn"

		-- ======================		Report:	Local faith production	=========================
		elseif iReconReportIDX == 5 then		
			local iTargetCityFaith = pTargetCity:GetYieldRate(YieldTypes.YIELD_FAITH)		-- Get city's yield of faith per turn
			iTargetCityFaith = AdjustCount(iTargetCityFaith, iReliabiltyIDX, false)			-- Adjust report results based on quality of report

			sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " generates " .. iTargetCityFaith .. " [ICON_PEACE] faith per turn"
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " generates " .. iTargetCityFaith .. " science per turn"
		
		-- ======================		Report:	Local happiness	=========================
		elseif iReconReportIDX == 6 then		
			local iTargetCityHappiness = pTargetCity:GetLocalHappiness()					-- Get city's happiness level
			iTargetCityHappiness = AdjustCount(iTargetCityHappiness, iReliabiltyIDX, true)	-- Adjust report results based on quality of report

			sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " has local happiness of [ICON_HAPPINESS_1] " .. iTargetCityHappiness
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " has local happiness of " .. iTargetCityHappiness	
	
		-- ======================		Report:	Local number of buildings	=========================
		elseif iReconReportIDX == 7 then								
			local iTargetCityBuildingCount = pTargetCity:GetNumBuildings()					-- Get city's total number of buildings
			iTargetCityBuildingCount = AdjustCount(iTargetCityBuildingCount, iReliabiltyIDX, false)	-- Adjust report results based on quality of report

			sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " has " .. iTargetCityBuildingCount .. " buildings"
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " has " .. iTargetCityBuildingCount .. " buildings"	
			
		-- ======================		Report:	Local number of wonders	=========================
		elseif iReconReportIDX == 8 then				
			local iTargetCityWonderCount = 0
			local sWonderType = "National"
			if math.random(1, 100) < 66 then
				iTargetCityWonderCount = pTargetCity:GetNumNationalWonders()				-- Get city's total number of buildings
			else
				iTargetCityWonderCount = pTargetCity:GetNumWorldWonders()					-- Get city's total number of buildings
				sWonderType = "World"
			end			
			iTargetCityWonderCount = AdjustCount(iTargetCityWonderCount, iReliabiltyIDX, false)	-- Adjust report results based on quality of report
			sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " has " .. iTargetCityWonderCount .. " " .. sWonderType .. " Wonders"
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " has " .. iTargetCityWonderCount .. " " .. sWonderType .. " Wonders"	
	
		-- ======================		Report:	Local list of world wonders	=========================     iReconReliabilityScore
		elseif iReconReportIDX == 9 then		
			sReconReport = sReconReport .. "The city of " .. pTargetCity:GetName() .. " has these World Wonders:[NEWLINE]"		-- Set up report header
			local iBuildingCount = 0														-- Track the number of buildings found - used if no buildings are found and for summary report
			for Building in GameInfo.Buildings() do											-- Cycle through all building types				
				if pTargetCity:GetNumBuilding(Building.ID) > 0 then							-- Does this city have this building?
					if GameInfo.BuildingClasses[GameInfo.Buildings[Building.ID].BuildingClass].MaxGlobalInstances > -1 then		-- World Wonder						
						local iAdjustedSpyCapability = 82 + (iUnitLevel * 3)				-- Higher level spies have better chance; 85% for Level 1, and 94% for Level 4																	
						if math.random(1, 100 ) < iAdjustedSpyCapability then
							local sWonderName = Locale.ConvertTextKey(GameInfo.Buildings[Building.ID].Description) or "Unknown Wonder"		-- Update report
							sReconReport = sReconReport .. "[ICON_BULLET]" .. sWonderName .. "[NEWLINE]"							
							iBuildingCount = iBuildingCount + 1								-- Bump building count
						end				
					end
				end
			end
			if iBuildingCount == 0 then														-- If no World wonders in pTargetCity, update report accordingly
				sReconReport = sReconReport .. "[ICON_BULLET]None[NEWLINE]"
			end					
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " has constructed " .. iBuildingCount .. " World Wonders"				

		-- ======================		Report:	Local list of national wonder =========================
		elseif iReconReportIDX == 10 then	
			sReconReport = sReconReport .. "The city of " .. pTargetCity:GetName() .. " has these National Wonders:[NEWLINE]"		-- Set up report header
			local iBuildingCount = 0														-- Track the number of buildings found - used if no buildings are found and for summary report
			for Building in GameInfo.Buildings() do											-- Cycle through all building types				
				if pTargetCity:GetNumBuilding(Building.ID) > 0 then							-- Does this city have this building?
					if GameInfo.BuildingClasses[GameInfo.Buildings[Building.ID].BuildingClass].MaxPlayerInstances > -1 then		-- National Wonder
						local iAdjustedSpyCapability = 80 + (iUnitLevel * 3)				-- Higher level spies have better chance; 83% for Level 1, and 92% for Level 4																	
						if math.random(1, 100 ) < iAdjustedSpyCapability then
							local sWonderName = Locale.ConvertTextKey(GameInfo.Buildings[Building.ID].Description) or "Unknown Wonder"		-- Update report
							sReconReport = sReconReport .. "[ICON_BULLET]" .. sWonderName .. "[NEWLINE]"							
							iBuildingCount = iBuildingCount + 1								-- Bump building count
						end				
					end
				end
			end
			if iBuildingCount == 0 then														-- If no national wonders in pTargetCity, update report accordingly
				sReconReport = sReconReport .. "[ICON_BULLET]None[NEWLINE]"
			end					
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " has constructed " .. iBuildingCount .. " National Wonders"	

		-- ======================		Report:	Local list of buildings	=========================
		elseif iReconReportIDX == 11 then
			sReconReport = sReconReport .. "The city of " .. pTargetCity:GetName() .. " has these buildings:[NEWLINE]"		-- Set up report header
			local iBuildingCount = 0														-- Track the number of buildings found - used if no buildings are found and for summary report
			for Building in GameInfo.Buildings() do											-- Cycle through all building types				
				if pTargetCity:GetNumBuilding(Building.ID) > 0 then							-- Does this city have this building?
					if (GameInfo.BuildingClasses[GameInfo.Buildings[Building.ID].BuildingClass].MaxGlobalInstances == -1) and			-- It's not a world wonder AND
							(GameInfo.BuildingClasses[GameInfo.Buildings[Building.ID].BuildingClass].MaxPlayerInstances  == -1) then		-- it's not a national wonder				
						local iAdjustedSpyCapability = 80 + (iUnitLevel * 3)				-- Higher level spies have better chance; 83% for Level 1, and 92% for Level 4																	
						if math.random(1, 100 ) < iAdjustedSpyCapability then
							local sBuildingName = Locale.ConvertTextKey(GameInfo.Buildings[Building.ID].Description) or "Unknown Building"		-- Update report
							sReconReport = sReconReport .. "[ICON_BULLET]" .. sBuildingName .. "[NEWLINE]"							
							iBuildingCount = iBuildingCount + 1								-- Bump building count
						end				
					end
				end
			end
			if iBuildingCount == 0 then														-- If no buildings in pTargetCity, update report accordingly
				sReconReport = sReconReport .. "[ICON_BULLET]None[NEWLINE]"
			end					
			sReconSummary = "The city of " .. pTargetCity:GetName() .. " has constructed " .. iBuildingCount .. " buildings"	

		-- ======================		Report:	Current production	=========================
		elseif iReconReportIDX == 12 then	
			local iBuildingProduced = pTargetCity:GetProductionBuilding()					-- If building is being produced, value == 1
			local iProjectProduced = pTargetCity:GetProductionProject()						-- If Project is being produced, value == 1
			local iUnitProduced = pTargetCity:GetProductionUnit()							-- If Unit is being produced, value == 1
			
			local iHammersInvested = pTargetCity:GetProduction()							-- How much production already completed?
			iHammersInvested = AdjustCount(iHammersInvested, iReliabiltyIDX, true)			-- Adjust report results based on quality of report			
			local iHammersNeeded = pTargetCity:GetProductionNeeded()						-- How much production already completed?
			local iPercentCompleted = math.floor((iHammersInvested / iHammersNeeded) * 100)		-- Get percent completed based on "adjusted" completion

			if iBuildingProduced ~= -1 then													-- City is producing a building
				local sBuildingName = Locale.ConvertTextKey(GameInfo.Buildings[iBuildingProduced].Description) or "Unknown Building"		-- Building description
				sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " is constructing:[NEWLINE]          " .. sBuildingName .. " (" .. iPercentCompleted .. "% completed)"
				sReconSummary = "The city of " .. pTargetCity:GetName() .. " is constructing: " .. sBuildingName .. " (" .. iPercentCompleted .. "%)"
			elseif iProjectProduced ~= -1 then												-- City is producing a project
				local sProjectName = Locale.ConvertTextKey(GameInfo.Projects[iProjectProduced].Description) or "Unknown Project"
				sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " is contributing to Project:[NEWLINE]          " .. sProjectName 		
				sReconSummary = "The city of " .. pTargetCity:GetName() .. " is working on Project: " .. sProjectName 
			elseif iUnitProduced ~= -1 then													-- City is producing a unit
				local sUnitName = Locale.ConvertTextKey(GameInfo.Units[iUnitProduced].Description) or "Unknown Unit"
				sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " is training:[NEWLINE]          " .. sUnitName .. " (" .. iPercentCompleted .. "% completed)"				
				sReconSummary = "The city of " .. pTargetCity:GetName() .. " is training: " .. sUnitName .. " (" .. iPercentCompleted .. "%)"
			else																			-- City producing something that cannot be determined
				sReconReport = sReconReport .. "[ICON_BULLET]The city of " .. pTargetCity:GetName() .. " is not constructing a Building, Project, or Unit"
				sReconSummary = "The city of " .. pTargetCity:GetName() .. " is producing an unknown item"	
			end						
		else
			print("Unknown local report - problem....  iReconReportIDX: " .. iReconReportIDX)
		end


-- =============================================================================================================================================================================
	else		-- Global (civ-wide) reports	


					
		-- ======================		Report: Current Era		=========================
		if iReconReportIDX == 1 then					
			local sEraName = "Unknown Era"
			local sArticleName = "an"
			local iTargetPlayerEra = pTargetPlayer:GetCurrentEra()							-- Get target civ's current era
			local iAdjustedSpyCapability = 80 + (iUnitLevel * 3)							-- Higher level spies have better chance; 83% for Level 1, and 92% for Level 4
			if (iTargetPlayerEra) and (math.random(1, 100 ) < iAdjustedSpyCapability) then		-- Is in a valid era and about 90% of the time, get the tech report right; otherwise "unknown era"			
				sEraName = Locale.ConvertTextKey(GameInfo.Eras[iTargetPlayerEra].Description) or "Unknown Era"		-- Era description
				sArticleName = "the"
			end
			sReconReport = sReconReport .. "[ICON_BULLET]The " .. sTargetPlayerAdjective .. " civilization is in " .. sArticleName .. " " .. sEraName
			sReconSummary = "The " .. sTargetPlayerAdjective .. " civilization is in " .. sArticleName .. " " .. sEraName

		-- ======================		Report:	Tech being researched	=========================
		elseif iReconReportIDX == 2 then	
			local sTechName = "an Unknown Tech"												-- Default tech string to unknown
			local iTargetPlayerTechRearching = pTargetPlayer:GetCurrentResearch()			-- Get target civ's current tech research
			local iAdjustedSpyCapability = 80 + (iUnitLevel * 3)							-- Higher level spies have better chance; 83% for Level 1, and 92% for Level 4						
			if (iTargetPlayerTechRearching >= 0) and (math.random(1, 100 ) < iAdjustedSpyCapability) then		-- Is researching a valid tech and about 90% of the time, get the tech report right; otherwise "unknown tech"
				sTechName = Locale.ConvertTextKey(GameInfo.Technologies[iTargetPlayerTechRearching].Description)		-- Tech description	
			end
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " is researching " .. sTechName
			sReconSummary = sTargetPlayerShortDescription .. " is researching " .. sTechName
			
		-- ======================		Report:	Number of researched techs	=========================
		elseif iReconReportIDX == 3 then
			local pTargetTeam = Teams[pTargetPlayer:GetTeam()]								-- Get pointer to target team
			local iActivePlayerTechCount = pActivePlayerTeam:GetTeamTechs():GetNumTechsKnown()		-- Get total techs active player has discovered
			local iTargetTeamTechCount = pTargetTeam:GetTeamTechs():GetNumTechsKnown()		-- Get total techs discovered by target team/player
			iTargetTeamTechCount = AdjustCount(iTargetTeamTechCount, iReliabiltyIDX, false)	-- Adjust report results based on quality of report					
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " has researched " .. iTargetTeamTechCount .. " technologies"			
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerShortDescription .. " has " .. iActivePlayerTechCount .. " techs"						
			sReconSummary = sTargetPlayerShortDescription .. " has researched " .. iTargetTeamTechCount .. " technologies"	

		-- ======================		Report:	Tech research progress	=========================
		elseif iReconReportIDX == 4 then													-- Note: displayed tech is always correct; turns until completion is variable					
			local iTargetPlayerTechRearching = pTargetPlayer:GetCurrentResearch()			-- Returns Tech ID of currently researched tech							
			local iResearchTurnsLeft = pTargetPlayer:GetResearchTurnsLeft(iTargetPlayerTechRearching, true)		-- Returns number of turns left until discovery
			iResearchTurnsLeft = AdjustCount(iResearchTurnsLeft, iReliabiltyIDX, false)		-- Adjust report results based on quality of report					

			if iTargetPlayerTechRearching >= 0 then
				local sTechName = Locale.ConvertTextKey(GameInfo.Technologies[iTargetPlayerTechRearching].Description)				
				sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " is researching " .. sTechName
				sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]Estimated turns until discovery: " .. iResearchTurnsLeft
				sReconSummary = sTargetPlayerAdjective .. " will discover " .. sTechName .. " in " .. iResearchTurnsLeft .. " turns"
			else
				sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " is researching an unknown tech"
				sReconSummary = sTargetPlayerShortDescription .. " is researching an unknown tech"
			end			

		-- ======================		Report:	Policy progress	=========================
		elseif iReconReportIDX == 5 then			
			local iCultureOnHand = pTargetPlayer:GetJONSCulture()							-- Get current excess culture
			local iCulturePerTurn = pTargetPlayer:GetTotalJONSCulturePerTurn()				-- Get how much culture is being generated each turn
			local iCultureNextPolicyCost = pTargetPlayer:GetNextPolicyCost()				-- Determine cost of next policy
			local iTurnsRemaining =  math.floor((iCultureNextPolicyCost - iCultureOnHand) / iCulturePerTurn)		-- Basec on culture per turn, figure out how long until next policy is available
			iTurnsRemaining = AdjustCount(iTurnsRemaining, iReliabiltyIDX, false)			-- Adjust report results based on quality of report								
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " will adopt its next social policy in " .. iTurnsRemaining .. " turns"
			sReconSummary = sTargetPlayerShortDescription .. " will adopt its next social policy in " .. iTurnsRemaining .. " turns"

		-- ======================		Report:	Golden Age progress	=========================
		elseif iReconReportIDX == 6 then				
			local iGATurns = pTargetPlayer:GetGoldenAgeTurns()								-- Get the number of GA turns left for target civ; if > 0 then they're in a GA
			if iGATurns > 0 then				
				iGATurns = AdjustCount(iGATurns, iReliabiltyIDX, false)						-- Adjust report results based on quality of report					
				sReconReport = sReconReport .. sTargetPlayerShortDescription .. " is enjoying a Golden Age for " .. iGATurns .. " more turns"
				sReconSummary = sTargetPlayerShortDescription .. " is enjoying a Golden Age for " .. iGATurns .. " more turns"
			else																			-- Target civ is not in a GA
				local iGAMeter = pTargetPlayer:GetGoldenAgeProgressMeter()					-- Get target civ progress to their next GA
				local iGAThreshold = pTargetPlayer:GetGoldenAgeProgressThreshold()			-- What is the threshold for that next GA
				iGAMeter = AdjustCount(iGAMeter, iReliabiltyIDX, false)						-- Adjust report results based on quality of report	
				iGAThreshold = AdjustCount(iGAThreshold, iReliabiltyIDX, false)				-- Adjust report results based on quality of report								
				sReconReport = sReconReport .. "The " .. sTargetPlayerAdjective .. " Golden Age meter looks like: " .. iGAMeter .. "/" .. iGAThreshold					
				sReconSummary =  "The " .. sTargetPlayerAdjective .. " Golden Age meter looks like: " .. iGAMeter .. "/" .. iGAThreshold	
			end			

		-- ======================		Report:	Wonder being built anywhere?	=========================
		elseif (iReconReportIDX == 7) or (iReconReportIDX == 8) then						-- 7 gives the wonder/location; 8 give wonder/location/progress
			local iNumWondersFound = 0	
			local tWonderInfo = {}		
			for pTargCity in pTargetPlayer:Cities() do
				local iBuildingProduced = pTargCity:GetProductionBuilding()					-- If building is being produced, value == 1
				if iBuildingProduced ~= -1 then													-- City is producing a building
					if (GameInfo.BuildingClasses[GameInfo.Buildings[iBuildingProduced].BuildingClass].MaxGlobalInstances ~= -1) or			-- It's a world wonder or
						(GameInfo.BuildingClasses[GameInfo.Buildings[iBuildingProduced].BuildingClass].MaxPlayerInstances  ~= -1) then		-- it's a national wonder
						local iAdjustedSpyCapability = 70 + (iUnitLevel * 3)							-- Higher level spies have better chance; 73% for Level 1, and 82% for Level 4																	
						if math.random(1, 100) < iAdjustedSpyCapability then							-- Successfully discovered building						
							iNumWondersFound = iNumWondersFound + 1										-- Increment total reported wonders counter
							local sBuildingName = Locale.ConvertTextKey(GameInfo.Buildings[iBuildingProduced].Description) or "Unknown Building"		-- Building description							
							local turnsleft = pTargCity:GetBuildingProductionTurnsLeft( iBuildingProduced )
							tWonderInfo[iNumWondersFound] = { pTargCity:GetName(), sBuildingName, turnsleft}						
						end
					end
				end
			end						
			if iNumWondersFound == 0 then
				sReconReport = sReconReport .. sTargetPlayerShortDescription .. " is not constructing any wonders"
				sReconSummary = sTargetPlayerShortDescription .. " is not constructing any wonders"
			else				
				local iIndex = math.random(1, #tWonderInfo)
				local sTurnCompletionReport = ""
				local sTurnCompletionSummary = ""				
				if iReconReportIDX == 8 then
					local iTurnsToComplete = tWonderInfo[iIndex][3]
					iTurnsToComplete = AdjustCount(iTurnsToComplete, iReliabiltyIDX, false)		-- Adjust report results based on quality of report	
					sTurnCompletionReport = "        Turns until Completion: " .. iTurnsToComplete
					sTurnCompletionSummary = " in " .. iTurnsToComplete .. " turns"
				end
				sReconReport = sReconReport .. sTargetPlayerShortDescription .. " is constructing the " .. tWonderInfo[iIndex][2] .. "[NEWLINE]Location: " .. tWonderInfo[iIndex][1] .. sTurnCompletionReport
				sReconSummary = sTargetPlayerShortDescription .. " is constructing the " .. tWonderInfo[iIndex][2] .. " near " .. tWonderInfo[iIndex][1] .. sTurnCompletionSummary
			end			

		-- ======================		Report:	Wonder progress?	=========================
		--elseif iReconReportIDX == 8 then

		-- ======================		Report:	Number of wonders already built anywhere	=========================
		elseif iReconReportIDX == 9 then

			local iWorldWonderCount = Game.GetNumWorldWonders()								-- Get total world wonders			
			iWorldWonderCount = AdjustCount(iWorldWonderCount, iReliabiltyIDX, false)		-- Adjust report results based on quality of report					

			sReconReport = sReconReport .. "[ICON_BULLET]Based on discussions with Operatives with other civilizations,"			
			sReconReport = sReconReport .. "[NEWLINE]       the estimated number of World Wonders is " .. iWorldWonderCount						
			sReconSummary = "The estimated number of completed World Wonders is " .. iWorldWonderCount

		-- ======================		Report: Number of unlocked policies	=========================
		elseif iReconReportIDX == 10 then
			local iActivePlayerPolicyCount = Players[Game.GetActivePlayer()]:GetNumPolicies()		-- Get active player total policies
			local iTargetPlayerPolicyCount = pTargetPlayer:GetNumPolicies()					-- Get total techs discovered by target team/player
			
			iTargetPlayerPolicyCount = AdjustCount(iTargetPlayerPolicyCount, iReliabiltyIDX, false)		-- Adjust report results based on quality of report					
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " has adopted " .. iTargetPlayerPolicyCount .. " social policies"			
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerShortDescription .. " has " .. iActivePlayerPolicyCount .. " policies"						
			sReconSummary = sTargetPlayerShortDescription .. " has adopted " .. iTargetPlayerPolicyCount .. " policies"	

		-- ======================		Report:	Global gold production	=========================
		elseif iReconReportIDX == 11 then
			local iGoldPerTurn = pTargetPlayer:CalculateGoldRate()							-- Get target civ gold per turn
			iGoldPerTurn = AdjustCount(iGoldPerTurn, iReliabiltyIDX, true)					-- Adjust report results based on quality of report
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " generates " .. iGoldPerTurn .. " [ICON_GOLD] Gold per turn"						
			sReconSummary = sTargetPlayerShortDescription .. " generates " .. iGoldPerTurn .. " Gold per turn"	

		-- ======================		Report:	Global science production	=========================
		elseif iReconReportIDX == 12 then
			local iSciencePerTurn = pTargetPlayer:GetScience()								-- Get target civ gold per turn
			iSciencePerTurn = AdjustCount(iSciencePerTurn, iReliabiltyIDX, true)			-- Adjust report results based on quality of report				
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " generates " .. iSciencePerTurn .. " [ICON_RESEARCH] Science per turn"						
			sReconSummary = sTargetPlayerShortDescription .. " generates " .. iSciencePerTurn .. " Science per turn"	

		-- ======================		Report:	Global culture production	=========================
		elseif iReconReportIDX == 13 then
			local iCulturePerTurn = pTargetPlayer:GetTotalJONSCulturePerTurn()				-- Get how much culture is being generated each turn			
			iCulturePerTurn = AdjustCount(iCulturePerTurn, iReliabiltyIDX, false)			-- Adjust report results based on quality of report
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " generates " .. iCulturePerTurn .. " [ICON_CULTURE] Culture per turn"						
			sReconSummary = sTargetPlayerShortDescription .. " generates " .. iCulturePerTurn .. " Culture per turn"	

		-- ======================		Report:	Global faith production	=========================
		elseif iReconReportIDX == 14 then
			local iFaithPerTurn = pTargetPlayer:GetTotalFaithPerTurn()						-- Get how much culture is being generated each turn			
			iFaithPerTurn = AdjustCount(iFaithPerTurn, iReliabiltyIDX, false)				-- Adjust report results based on quality of report
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " generates " .. iFaithPerTurn .. " [ICON_PEACE] Faith per turn"						
			sReconSummary = sTargetPlayerShortDescription .. " generates " .. iFaithPerTurn .. " Faith per turn"	

		-- ======================		Report:	Global happiness	=========================
		elseif iReconReportIDX == 15 then
			local iHappiness = pTargetPlayer:GetExcessHappiness()							-- Get current excess happiness
			iHappiness = AdjustCount(iHappiness, iReliabiltyIDX, true)						-- Adjust report results based on quality of report			
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerAdjective .. " happiness is estimated at " .. iHappiness
			sReconSummary = sTargetPlayerAdjective .. " happiness is estimated at " .. iHappiness

		-- ======================		Report:	Military might	=========================
		elseif iReconReportIDX == 16 then

			local iTargetMilMight = pTargetPlayer:GetMilitaryMight()						-- Get target military might setting
			local iFriendlyMilMight = Players[Game.GetActivePlayer()]:GetMilitaryMight()	-- Get friendly military might setting
			iTargetMilMight = AdjustCount(iTargetMilMight, iReliabiltyIDX, false)			-- Adjust report results based on quality of report					
			
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerAdjective .. " military strength assessed at " .. iTargetMilMight			
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerAdjective .. " strength assessed at " .. iFriendlyMilMight
			sReconSummary = sTargetPlayerAdjective .. " military strength assessed at " .. iTargetMilMight	.. " (" .. sActivePlayerAdjective .. " might is " .. iFriendlyMilMight
			
		-- ======================		Report:	Total military unit count	=========================
		elseif iReconReportIDX == 17 then
			local iTargetUnitCount = pTargetPlayer:GetNumMilitaryUnits()					-- Get target military unit count
			iTargetUnitCount = AdjustCount(iTargetUnitCount, iReliabiltyIDX, false)			-- Adjust report results based on quality of report			
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " has " .. iTargetUnitCount .. " military units"
			sReconSummary = sTargetPlayerShortDescription .. " has " .. iTargetUnitCount .. " military units"		
		
		-- ======================		Report:	Total unit out of supply count	=========================
		elseif iReconReportIDX == 18 then
			local iTargetUnitCount = pTargetPlayer:GetNumUnitsOutOfSupply()					-- Get target units out of supply
			iTargetUnitCount = AdjustCount(iTargetUnitCount, iReliabiltyIDX, false)			-- Adjust report results based on quality of report			
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " has " .. iTargetUnitCount .. " units out of supply"
			sReconSummary = sTargetPlayerShortDescription .. " has " .. iTargetUnitCount .. " units out of supply"		
		
		-- ======================		Report:	Total nuclear unit count	=========================
		elseif iReconReportIDX == 19 then
			local iTargetUnitCount = pTargetPlayer:GetNumNukeUnits()						-- Get total number of target nuke units
			iTargetUnitCount = AdjustCount(iTargetUnitCount, iReliabiltyIDX, false)			-- Adjust report results based on quality of report						
			local iTargetPlayerEra = pTargetPlayer:GetCurrentEra()							-- Get target civ's current era
			if iTargetPlayerEra < GameInfo.Eras["ERA_MODERN"].ID then iTargetUnitCount = 0 end	-- If target's era is earlier than Modern, no possibility of a mistake like this			
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " has " .. iTargetUnitCount .. " nuclear units"
			sReconSummary = sTargetPlayerShortDescription .. " has " .. iTargetUnitCount .. " nuclear units"				
		
		-- ======================		Report:	Total spy count	=========================
		elseif iReconReportIDX == 20 then
			local tTargetSpyInfo = pTargetPlayer:GetEspionageSpies()						-- Get Spy Table data
			local iSpyCount = #tTargetSpyInfo												-- Count rows for spy count
			iSpyCount = AdjustCount(iSpyCount, iReliabiltyIDX, false)						-- Adjust report results based on quality of report; neg numbers NOT ok			
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " has " .. iSpyCount .. " spies"
			sReconSummary = sTargetPlayerShortDescription .. " has " .. iSpyCount .. " spies"

		-- ======================		Report:	Spy locations	=========================
		elseif iReconReportIDX == 21 then
			local tTargetSpyInfo = pTargetPlayer:GetEspionageSpies()						-- Get Spy Table data
			local iSpyCount = #tTargetSpyInfo or 0											-- Count rows for spy count
			local iSpiesDiscovered = 0
			local sSpyMessageBody = ""
			local sSpyMessageSummary = ""
			for i,v in ipairs(tTargetSpyInfo) do											-- Look through all spies in the table
				local iAdjustedSpyCapability = 50 + (iUnitLevel * 5)						-- Higher level spies have better chance; 53% for Level 1, and 70% for Level 4																	
				if math.random(1, 100) < iAdjustedSpyCapability then						-- Successfully discovered data concerning this spy				
					local pSpyCity															-- Prep city instance for spy city location
					local sSpyCityName = "an Unknown City"										-- Name for spy city
					local sSpyName = Locale.ConvertTextKey(v.Name) or "Unknown"				-- Get spy's name
					local sSpyRank = Locale.ConvertTextKey(v.Rank) or "Unknown"				-- Get spy's rank
					local pSpyPlot = Map.GetPlot(v.CityX, v.CityY)							-- Get plot of spy's location
					if pSpyPlot ~= nil then													-- If plot is valid, then
						local pSpyCity = pSpyPlot:GetPlotCity()								-- Create city instance for the spy plot
						if pSpyCity ~= nil then												-- If city is valid, then
							sSpyCityName = pSpyCity:GetName()								-- Get the city's name
						end
					end								
					iSpiesDiscovered = iSpiesDiscovered + 1							
					sSpyMessageBody = sSpyMessageBody .. "[ICON_BULLET]" .. sSpyName .. " (" .. sSpyRank .. ") is located near " .. sSpyCityName .. "[NEWLINE]"
					sSpyMessageSummary = sSpyMessageSummary .. "   " .. sSpyCityName
				end
			end
			if iSpiesDiscovered == 0 then sSpyMessageBody = "[NEWLINE]Unknown" end
			sReconReport = sReconReport .. sTargetPlayerAdjective .. " Spy Disposition:[NEWLINE]" .. sSpyMessageBody
			sReconSummary = sTargetPlayerShortDescription .. " has " .. iSpyCount .. " spies located at: " .. sSpyMessageSummary

		-- ======================		Report:	Building count	=========================
		elseif iReconReportIDX == 22 then

			local iTargetBuildingCount = 0
			for pTargCity in pTargetPlayer:Cities() do
				iTargetBuildingCount = iTargetBuildingCount + pTargCity:GetNumBuildings()
			end
			iTargetBuildingCount = AdjustCount(iTargetBuildingCount, iReliabiltyIDX, false)	-- Adjust report results based on quality of report; neg numbers NOT ok			
			
			local iActivePlayerBuildingCount = 0
			for pActivePlayerCity in Players[Game.GetActivePlayer()]:Cities() do
				iActivePlayerBuildingCount = iActivePlayerBuildingCount + pActivePlayerCity:GetNumBuildings()
			end			
			
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " has " .. iTargetBuildingCount .. " buildings completed"
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerShortDescription .. " has " .. iActivePlayerBuildingCount .. " buildings"
			sReconSummary = sTargetPlayerShortDescription .. " has " .. iTargetBuildingCount .. " buildings, " .. sActivePlayerShortDescription .. " has " .. iActivePlayerBuildingCount
							
		-- ======================		Report:	Building maintenance cost	=========================
		elseif iReconReportIDX == 23 then		
			local iActivePlayerBuildingCost = Players[Game.GetActivePlayer()]:GetBuildingGoldMaintenance()		-- Get active player total policies
			local iTargetBuildingCost = pTargetPlayer:GetBuildingGoldMaintenance()			-- Get Spy Table data			
			iTargetBuildingCost = AdjustCount(iTargetBuildingCost, iReliabiltyIDX, false)						-- Adjust report results based on quality of report; neg numbers NOT ok			
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerShortDescription .. " spends " .. iTargetBuildingCost .. " [ICON_GOLD] Gold for building maintenance"
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerShortDescription .. " spends " .. iActivePlayerBuildingCost .. " [ICON_GOLD] Gold"
			sReconSummary = sTargetPlayerShortDescription .. " spends " .. iTargetBuildingCost .. " Gold for building maintenance, " .. sActivePlayerShortDescription .. " spends " .. iActivePlayerBuildingCost		
		
		-- ======================		Report:	Score: land	=========================
		elseif iReconReportIDX == 24 then				
			local iActivePlayerScore = Players[Game.GetActivePlayer()]:GetScore()			-- Get active player total score
			local iTargetPlayerScore = pTargetPlayer:GetScore()								-- Get target player total score

			local iActivePlayerLandScore = Players[Game.GetActivePlayer()]:GetScoreFromLand()		-- Get active player land score
			local iTargetPlayerLandScore = pTargetPlayer:GetScoreFromLand()					-- Get target player land score
			iTargetPlayerLandScore = AdjustCount(iTargetPlayerLandScore, iReliabiltyIDX, false)		-- Adjust report results based on quality of report; neg numbers NOT ok			
			
			local iScorePercentage = math.floor((iTargetPlayerLandScore/iTargetPlayerScore) * 100)	
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerAdjective .. " land score is " .. iTargetPlayerLandScore .. ", from " .. iTargetPlayerScore .. " total"
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of their total score"
			
			iScorePercentage = math.floor((iActivePlayerLandScore/iActivePlayerScore) * 100)
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerAdjective .. " land score is " .. iActivePlayerLandScore .. " from " .. iActivePlayerScore
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of your total score"
			
			sReconSummary = sTargetPlayerAdjective .. " land score is " .. iTargetPlayerLandScore .. " out of " .. iTargetPlayerScore .. " total, " .. sActivePlayerAdjective .. " land score is " .. iActivePlayerLandScore ..  " from " .. iActivePlayerScore					
		
		-- ======================		Report:	Score: cities	=========================
		elseif iReconReportIDX == 25 then
		local iActivePlayerScore = Players[Game.GetActivePlayer()]:GetScore()			-- Get active player total score
			local iTargetPlayerScore = pTargetPlayer:GetScore()								-- Get target player total score

			local iActivePlayerLandScore = Players[Game.GetActivePlayer()]:GetScoreFromLand()		-- Get active player land score
			local iTargetPlayerLandScore = pTargetPlayer:GetScoreFromLand()					-- Get target player land score
			iTargetPlayerLandScore = AdjustCount(iTargetPlayerLandScore, iReliabiltyIDX, false)		-- Adjust report results based on quality of report; neg numbers NOT ok			
			
			local iScorePercentage = math.floor((iTargetPlayerLandScore/iTargetPlayerScore) * 100)	
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerAdjective .. " land score is " .. iTargetPlayerLandScore .. ", from " .. iTargetPlayerScore .. " total"
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of their total score"
			
			iScorePercentage = math.floor((iActivePlayerLandScore/iActivePlayerScore) * 100)
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerAdjective .. " land score is " .. iActivePlayerLandScore .. " from " .. iActivePlayerScore
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of your total score"
			
			sReconSummary = sTargetPlayerAdjective .. " land score is " .. iTargetPlayerLandScore .. " out of " .. iTargetPlayerScore .. " total, " .. sActivePlayerAdjective .. " land score is " .. iActivePlayerLandScore ..  " from " .. iActivePlayerScore					
			
		-- ======================		Report:	Score: population	=========================
		elseif iReconReportIDX == 26 then
			local iActivePlayerScore = Players[Game.GetActivePlayer()]:GetScore()			-- Get active player total score
			local iTargetPlayerScore = pTargetPlayer:GetScore()								-- Get target player total score

			local iActivePlayerPopScore = Players[Game.GetActivePlayer()]:GetScoreFromPopulation()		-- Get active player population score
			local iTargetPlayerPopScore = pTargetPlayer:GetScoreFromPopulation()			-- Get target player population score
			iTargetPlayerPopScore = AdjustCount(iTargetPlayerPopScore, iReliabiltyIDX, false)		-- Adjust report results based on quality of report; neg numbers NOT ok			
			
			local iScorePercentage = math.floor((iTargetPlayerPopScore/iTargetPlayerScore) * 100)	
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerAdjective .. " population score is " .. iTargetPlayerPopScore .. ", from " .. iTargetPlayerScore .. " total"
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of their total score"
			
			iScorePercentage = math.floor((iActivePlayerPopScore/iActivePlayerScore) * 100)
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerAdjective .. " population score is " .. iActivePlayerPopScore .. " from " .. iActivePlayerScore
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of your total score"
			
			sReconSummary = sTargetPlayerAdjective .. " population score is " .. iTargetPlayerPopScore .. " out of " .. iTargetPlayerScore .. " total, " .. sActivePlayerAdjective .. " population score is " .. iActivePlayerPopScore ..  " from " .. iActivePlayerScore					
			
		-- ======================		Report:	Score: wonders	=========================
		elseif iReconReportIDX == 27 then
			local iActivePlayerScore = Players[Game.GetActivePlayer()]:GetScore()			-- Get active player total score
			local iTargetPlayerScore = pTargetPlayer:GetScore()								-- Get target player total score

			local iActivePlayerWonderScore = Players[Game.GetActivePlayer()]:GetScoreFromWonders()		-- Get active player wonder score
			local iTargetPlayerWonderScore = pTargetPlayer:GetScoreFromWonders()				-- Get target player wonder score
			iTargetPlayerWonderScore = AdjustCount(iTargetPlayerWonderScore, iReliabiltyIDX, false)		-- Adjust report results based on quality of report; neg numbers NOT ok			
			
			local iScorePercentage = math.floor((iTargetPlayerWonderScore/iTargetPlayerScore) * 100)	
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerAdjective .. " Wonders score is " .. iTargetPlayerWonderScore .. ", from " .. iTargetPlayerScore .. " total"
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of their total score"
			
			iScorePercentage = math.floor((iActivePlayerWonderScore/iActivePlayerScore) * 100)
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerAdjective .. " Wonders score is " .. iActivePlayerWonderScore .. " from " .. iActivePlayerScore
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of your total score"
			
			sReconSummary = sTargetPlayerAdjective .. " Wonders score is " .. iTargetPlayerWonderScore .. " out of " .. iTargetPlayerScore .. " total, " .. sActivePlayerAdjective .. " Wonders score is " .. iActivePlayerWonderScore ..  " from " .. iActivePlayerScore					
			
		-- ======================		Report:	Score: tech	=========================
		elseif iReconReportIDX == 28 then
			local iActivePlayerScore = Players[Game.GetActivePlayer()]:GetScore()			-- Get active player total score
			local iTargetPlayerScore = pTargetPlayer:GetScore()								-- Get target player total score

			local iActivePlayerTechsScore = Players[Game.GetActivePlayer()]:GetScoreFromTechs()		-- Get active player techs score
			local iTargetPlayerTechsScore = pTargetPlayer:GetScoreFromTechs()					-- Get target player techs score
			iTargetPlayerTechsScore = AdjustCount(iTargetPlayerTechsScore, iReliabiltyIDX, false)		-- Adjust report results based on quality of report; neg numbers NOT ok			
			
			local iScorePercentage = math.floor((iTargetPlayerTechsScore/iTargetPlayerScore) * 100)	
			sReconReport = sReconReport .. "[ICON_BULLET]" .. sTargetPlayerAdjective .. " technology score is " .. iTargetPlayerTechsScore .. ", from " .. iTargetPlayerScore .. " total"
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of their total score"
			
			iScorePercentage = math.floor((iActivePlayerTechsScore/iActivePlayerScore) * 100)
			sReconReport = sReconReport .. "[NEWLINE][NEWLINE][ICON_BULLET]By comparison, " .. sActivePlayerAdjective .. " technology score is " .. iActivePlayerTechsScore .. " from " .. iActivePlayerScore
			sReconReport = sReconReport .. "[NEWLINE]    This represents " .. iScorePercentage .. "% of your total score"
			
			sReconSummary = sTargetPlayerAdjective .. " technology score is " .. iTargetPlayerTechsScore .. " out of " .. iTargetPlayerScore .. " total, " .. sActivePlayerAdjective .. " technology score is " .. iActivePlayerTechsScore ..  " from " .. iActivePlayerScore					
						
		-- ======================		Report:	Total Building Class Count	=========================
		elseif iReconReportIDX == 29 then			
			local iBuildingCount = 0														-- Track the number of buildings found - used if no buildings are found and for summary report
			local tBuildingData = {}														-- Table to hold building date (name, count)
			
			for Building in GameInfo.Buildings() do											-- Cycle through all building types							
				if (GameInfo.BuildingClasses[GameInfo.Buildings[Building.ID].BuildingClass].MaxGlobalInstances == -1) and			-- It's not a world wonder AND
						(GameInfo.BuildingClasses[GameInfo.Buildings[Building.ID].BuildingClass].MaxPlayerInstances  == -1) then		-- it's not a national wonder, then
					local iNumBuildings = pTargetPlayer:CountNumBuildings(Building.ID)		-- Get total number of this building type
					if iNumBuildings > 0 then	-- If civ has one or more of these buildings, then							
						local iAdjustedSpyCapability = 84 + (iUnitLevel * 3)				-- Higher level spies have better chance; 87% for Level 1, and 96% for Level 4																	
						if math.random(1, 100 ) < iAdjustedSpyCapability then				-- If so, it was "discovered" by recon														
							iNumBuildings = AdjustCount(iNumBuildings, iReliabiltyIDX, false)	-- Adjust report results based on quality of report; neg numbers NOT ok
							if iNumBuildings < 1 then iNumBuildings = 1 end					-- Ensure at least 1 on building in the list
							local sBuildingName = Locale.ConvertTextKey(GameInfo.Buildings[Building.ID].Description) or "Unknown Building"		-- Get the text building name														
							iBuildingCount = iBuildingCount + 1								-- Bump building count
							tBuildingData[iBuildingCount] = { [1] = sBuildingName, [2] = iNumBuildings }	-- Add data to table (names and "adjusted" building count)

							print("Count: " ..iBuildingCount .. "   name: " ..tBuildingData[iBuildingCount][1] .. "   count: " .. tBuildingData[iBuildingCount][2])

						end		
					end		
				end				
			end

			if #tBuildingData ~= 0 then
				local iIndex = math.random(1, #tBuildingData)									-- Randomly select the building type from the table		
				
				if tBuildingData[iIndex][2] > 1 then											-- If more than 1 building, then
					local sLastChar = string.sub(tBuildingData[iIndex][1], -1)					-- Get last character in string					
					if sLastChar == "y" then tBuildingData[iIndex][1] = tBuildingData[iIndex][1]:sub(1, -2) .. "ies"	-- If it is a "y" then append an "ies" to the name
					elseif sLastChar ~= "s" then tBuildingData[iIndex][1] = tBuildingData[iIndex][1] .. "s" end		-- If it is not an "s" then append an "s" to the name					
				end				
								
				sReconReport = sReconReport .. "In total, " .. sTargetPlayerShortDescription .. " has constructed " .. tBuildingData[iIndex][2] .. " " .. tBuildingData[iIndex][1]
				sReconSummary = sTargetPlayerShortDescription .. " has constructed " .. tBuildingData[iIndex][2] .. " " .. tBuildingData[iIndex][1] 
			else
				sReconReport = sReconReport .. sTargetPlayerShortDescription .. " has an unknown number of buildings"
				sReconSummary = sTargetPlayerShortDescription .. " has an unknown number of buildings"
			end
		else
			print("Unknown global report - problem....  iReconReportIDX: " .. iReconReportIDX)
		end
	end	

	sReconReport = sReconReport .. "[NEWLINE]" .. sMessageLength .. "[NEWLINE]Report Quality is assessed as '" .. g_tReliability_Adjective[iReliabiltyIDX] .. "'[NEWLINE]"
	sReconSummary = sReconSummary .. "   (" .. g_tReliability_Adjective[iReliabiltyIDX] .. ")"
	
	return sReconHeader, sReconReport, sReconSummary

end